from enum import Enum


# noinspection SpellCheckingInspection
class AnalogDigitalMode(Enum):
	"""2 Members, ANALog ... DIGital"""
	ANALog = 0
	DIGital = 1


# noinspection SpellCheckingInspection
class ArbClockMode(Enum):
	"""2 Members, MSAMple ... SAMPle"""
	MSAMple = 0
	SAMPle = 1


# noinspection SpellCheckingInspection
class ArbLevMode(Enum):
	"""2 Members, HIGHest ... UNCHanged"""
	HIGHest = 0
	UNCHanged = 1


# noinspection SpellCheckingInspection
class ArbMultCarrCresMode(Enum):
	"""3 Members, MAX ... OFF"""
	MAX = 0
	MIN = 1
	OFF = 2


# noinspection SpellCheckingInspection
class ArbMultCarrLevRef(Enum):
	"""2 Members, PEAK ... RMS"""
	PEAK = 0
	RMS = 1


# noinspection SpellCheckingInspection
class ArbMultCarrSigDurMod(Enum):
	"""4 Members, LCM ... USER"""
	LCM = 0
	LONG = 1
	SHORt = 2
	USER = 3


# noinspection SpellCheckingInspection
class ArbMultCarrSpacMode(Enum):
	"""2 Members, ARBitrary ... EQUidistant"""
	ARBitrary = 0
	EQUidistant = 1


# noinspection SpellCheckingInspection
class ArbSegmNextSourIntNext(Enum):
	"""2 Members, INTernal ... NEXT"""
	INTernal = 0
	NEXT = 1


# noinspection SpellCheckingInspection
class ArbSignType(Enum):
	"""4 Members, AWGN ... SINE"""
	AWGN = 0
	CIQ = 1
	RECT = 2
	SINE = 3


# noinspection SpellCheckingInspection
class ArbTrigSegmModeNoEhop(Enum):
	"""4 Members, NEXT ... SEQuencer"""
	NEXT = 0
	NSEam = 1
	SAME = 2
	SEQuencer = 3


# noinspection SpellCheckingInspection
class ArbWaveSegmClocMode(Enum):
	"""3 Members, HIGHest ... USER"""
	HIGHest = 0
	UNCHanged = 1
	USER = 2


# noinspection SpellCheckingInspection
class ArbWaveSegmMarkMode(Enum):
	"""2 Members, IGNore ... TAKE"""
	IGNore = 0
	TAKE = 1


# noinspection SpellCheckingInspection
class ArbWaveSegmPowMode(Enum):
	"""2 Members, ERMS ... UNCHanged"""
	ERMS = 0
	UNCHanged = 1


# noinspection SpellCheckingInspection
class ArbWaveSegmRest(Enum):
	"""5 Members, MRK1 ... OFF"""
	MRK1 = 0
	MRK2 = 1
	MRK3 = 2
	MRK4 = 3
	OFF = 4


# noinspection SpellCheckingInspection
class AttMode(Enum):
	"""3 Members, APASsive ... FIXed"""
	APASsive = 0
	AUTO = 1
	FIXed = 2


# noinspection SpellCheckingInspection
class BbImpOptMode(Enum):
	"""4 Members, FAST ... UCORrection"""
	FAST = 0
	QHIGh = 1
	QHTable = 2
	UCORrection = 3


# noinspection SpellCheckingInspection
class BbinSampRateMode(Enum):
	"""2 Members, DIN ... USER"""
	DIN = 0
	USER = 1


# noinspection SpellCheckingInspection
class BbSystemConfiguration(Enum):
	"""2 Members, AFETracking ... STANdard"""
	AFETracking = 0
	STANdard = 1


# noinspection SpellCheckingInspection
class ByteOrder(Enum):
	"""2 Members, NORMal ... SWAPped"""
	NORMal = 0
	SWAPped = 1


# noinspection SpellCheckingInspection
class CalDataMode(Enum):
	"""2 Members, CUSTomer ... FACTory"""
	CUSTomer = 0
	FACTory = 1


# noinspection SpellCheckingInspection
class CalPowDetAtt(Enum):
	"""4 Members, HIGH ... OFF"""
	HIGH = 0
	LOW = 1
	MED = 2
	OFF = 3


# noinspection SpellCheckingInspection
class CfrAlgo(Enum):
	"""1 Members, CLFiltering ... CLFiltering"""
	CLFiltering = 0


# noinspection SpellCheckingInspection
class CfrFiltMode(Enum):
	"""2 Members, ENHanced ... SIMPle"""
	ENHanced = 0
	SIMPle = 1


# noinspection SpellCheckingInspection
class ClockSourceB(Enum):
	"""3 Members, AINTernal ... INTernal"""
	AINTernal = 0
	EXTernal = 1
	INTernal = 2


# noinspection SpellCheckingInspection
class ClocOutpMode(Enum):
	"""2 Members, BIT ... SYMBol"""
	BIT = 0
	SYMBol = 1


# noinspection SpellCheckingInspection
class ClocSourBb(Enum):
	"""4 Members, AINTernal ... INTernal"""
	AINTernal = 0
	COUPled = 1
	EXTernal = 2
	INTernal = 3


# noinspection SpellCheckingInspection
class ClocSyncMode(Enum):
	"""4 Members, DIIN ... SECondary"""
	DIIN = 0
	NONE = 1
	PRIMary = 2
	SECondary = 3


# noinspection SpellCheckingInspection
class Colour(Enum):
	"""4 Members, GREen ... YELLow"""
	GREen = 0
	NONE = 1
	RED = 2
	YELLow = 3


# noinspection SpellCheckingInspection
class DataSour(Enum):
	"""11 Members, DLISt ... ZERO"""
	DLISt = 0
	ONE = 1
	PATTern = 2
	PN11 = 3
	PN15 = 4
	PN16 = 5
	PN20 = 6
	PN21 = 7
	PN23 = 8
	PN9 = 9
	ZERO = 10


# noinspection SpellCheckingInspection
class DevExpFormat(Enum):
	"""4 Members, CGPRedefined ... XML"""
	CGPRedefined = 0
	CGUSer = 1
	SCPI = 2
	XML = 3


# noinspection SpellCheckingInspection
class DexchExtension(Enum):
	"""2 Members, CSV ... TXT"""
	CSV = 0
	TXT = 1


# noinspection SpellCheckingInspection
class DexchMode(Enum):
	"""2 Members, EXPort ... IMPort"""
	EXPort = 0
	IMPort = 1


# noinspection SpellCheckingInspection
class DexchSepCol(Enum):
	"""4 Members, COMMa ... TABulator"""
	COMMa = 0
	SEMicolon = 1
	SPACe = 2
	TABulator = 3


# noinspection SpellCheckingInspection
class DexchSepDec(Enum):
	"""2 Members, COMMa ... DOT"""
	COMMa = 0
	DOT = 1


# noinspection SpellCheckingInspection
class DmFilterA(Enum):
	"""18 Members, APCO25 ... SPHase"""
	APCO25 = 0
	C2K3x = 1
	COEQualizer = 2
	COF705 = 3
	COFequalizer = 4
	CONE = 5
	COSine = 6
	DIRac = 7
	ENPShape = 8
	EWPShape = 9
	GAUSs = 10
	LGAuss = 11
	LPASs = 12
	LPASSEVM = 13
	PGAuss = 14
	RCOSine = 15
	RECTangle = 16
	SPHase = 17


# noinspection SpellCheckingInspection
class DmTrigMode(Enum):
	"""5 Members, AAUTo ... SINGle"""
	AAUTo = 0
	ARETrigger = 1
	AUTO = 2
	RETRigger = 3
	SINGle = 4


# noinspection SpellCheckingInspection
class DpdPowRef(Enum):
	"""3 Members, ADPD ... SDPD"""
	ADPD = 0
	BDPD = 1
	SDPD = 2


# noinspection SpellCheckingInspection
class DpdShapeMode(Enum):
	"""3 Members, NORMalized ... TABLe"""
	NORMalized = 0
	POLYnomial = 1
	TABLe = 2


# noinspection SpellCheckingInspection
class FormData(Enum):
	"""2 Members, ASCii ... PACKed"""
	ASCii = 0
	PACKed = 1


# noinspection SpellCheckingInspection
class FormStatReg(Enum):
	"""4 Members, ASCii ... OCTal"""
	ASCii = 0
	BINary = 1
	HEXadecimal = 2
	OCTal = 3


# noinspection SpellCheckingInspection
class FreqStepMode(Enum):
	"""2 Members, DECimal ... USER"""
	DECimal = 0
	USER = 1


# noinspection SpellCheckingInspection
class IecDevId(Enum):
	"""2 Members, AUTO ... USER"""
	AUTO = 0
	USER = 1


# noinspection SpellCheckingInspection
class IecTermMode(Enum):
	"""2 Members, EOI ... STANdard"""
	EOI = 0
	STANdard = 1


# noinspection SpellCheckingInspection
class ImpG50G10K(Enum):
	"""2 Members, G10K ... G50"""
	G10K = 0
	G50 = 1


# noinspection SpellCheckingInspection
class InclExcl(Enum):
	"""2 Members, EXCLude ... INCLude"""
	EXCLude = 0
	INCLude = 1


# noinspection SpellCheckingInspection
class InputImpRf(Enum):
	"""3 Members, G10K ... G50"""
	G10K = 0
	G1K = 1
	G50 = 2


# noinspection SpellCheckingInspection
class Interpolation(Enum):
	"""3 Members, LINear ... POWer"""
	LINear = 0
	OFF = 1
	POWer = 2


# noinspection SpellCheckingInspection
class IqMode(Enum):
	"""2 Members, ANALog ... BASeband"""
	ANALog = 0
	BASeband = 1


# noinspection SpellCheckingInspection
class IqOutDispViaType(Enum):
	"""2 Members, LEVel ... PEP"""
	LEVel = 0
	PEP = 1


# noinspection SpellCheckingInspection
class IqOutEnvAdaption(Enum):
	"""3 Members, AUTO ... POWer"""
	AUTO = 0
	MANual = 1
	POWer = 2


# noinspection SpellCheckingInspection
class IqOutEnvDetrFunc(Enum):
	"""3 Members, F1 ... F3"""
	F1 = 0
	F2 = 1
	F3 = 2


# noinspection SpellCheckingInspection
class IqOutEnvEtRak(Enum):
	"""4 Members, ET1V2 ... USER"""
	ET1V2 = 0
	ET1V5 = 1
	ET2V0 = 2
	USER = 3


# noinspection SpellCheckingInspection
class IqOutEnvScale(Enum):
	"""2 Members, POWer ... VOLTage"""
	POWer = 0
	VOLTage = 1


# noinspection SpellCheckingInspection
class IqOutEnvShapeMode(Enum):
	"""6 Members, DETRoughing ... TABLe"""
	DETRoughing = 0
	LINear = 1
	OFF = 2
	POLYnomial = 3
	POWer = 4
	TABLe = 5


# noinspection SpellCheckingInspection
class IqOutEnvTerm(Enum):
	"""2 Members, GROund ... WIRE"""
	GROund = 0
	WIRE = 1


# noinspection SpellCheckingInspection
class IqOutEnvVrEf(Enum):
	"""2 Members, VCC ... VOUT"""
	VCC = 0
	VOUT = 1


# noinspection SpellCheckingInspection
class IqOutMode(Enum):
	"""3 Members, FIXed ... VATTenuated"""
	FIXed = 0
	VARiable = 1
	VATTenuated = 2


# noinspection SpellCheckingInspection
class IqOutType(Enum):
	"""2 Members, DIFFerential ... SINGle"""
	DIFFerential = 0
	SINGle = 1


# noinspection SpellCheckingInspection
class NetMode(Enum):
	"""2 Members, AUTO ... STATic"""
	AUTO = 0
	STATic = 1


# noinspection SpellCheckingInspection
class NoisAwgnDispModeRfBb(Enum):
	"""2 Members, BB ... RF"""
	BB = 0
	RF = 1


# noinspection SpellCheckingInspection
class NoisAwgnMode(Enum):
	"""3 Members, ADD ... ONLY"""
	ADD = 0
	CW = 1
	ONLY = 2


# noinspection SpellCheckingInspection
class NoisAwgnPowMode(Enum):
	"""3 Members, CN ... SN"""
	CN = 0
	EN = 1
	SN = 2


# noinspection SpellCheckingInspection
class NoisAwgnPowRefMode(Enum):
	"""2 Members, CARRier ... NOISe"""
	CARRier = 0
	NOISe = 1


# noinspection SpellCheckingInspection
class NormalInverted(Enum):
	"""2 Members, INVerted ... NORMal"""
	INVerted = 0
	NORMal = 1


# noinspection SpellCheckingInspection
class NumbersA(Enum):
	"""2 Members, _1 ... _2"""
	_1 = 0
	_2 = 1


# noinspection SpellCheckingInspection
class OpMode(Enum):
	"""2 Members, BBBYpass ... NORMal"""
	BBBYpass = 0
	NORMal = 1


# noinspection SpellCheckingInspection
class Parity(Enum):
	"""3 Members, EVEN ... ODD"""
	EVEN = 0
	NONE = 1
	ODD = 2


# noinspection SpellCheckingInspection
class PowAlcState(Enum):
	"""4 Members, _1 ... ON"""
	_1 = 0
	AUTO = 1
	OFF = 2
	ON = 3


# noinspection SpellCheckingInspection
class PowLevBehaviour(Enum):
	"""6 Members, AUTO ... USER"""
	AUTO = 0
	CVSWr = 1
	CWSWr = 2
	MONotone = 3
	UNINterrupted = 4
	USER = 5


# noinspection SpellCheckingInspection
class PowLevMode(Enum):
	"""4 Members, LOWDistortion ... USER"""
	LOWDistortion = 0
	LOWNoise = 1
	NORMal = 2
	USER = 3


# noinspection SpellCheckingInspection
class PowOutpPonMode(Enum):
	"""2 Members, OFF ... UNCHanged"""
	OFF = 0
	UNCHanged = 1


# noinspection SpellCheckingInspection
class PowRfOffMode(Enum):
	"""2 Members, FIXed ... MAX"""
	FIXed = 0
	MAX = 1


# noinspection SpellCheckingInspection
class PowSensWithUndef(Enum):
	"""9 Members, SENS1 ... UNDefined"""
	SENS1 = 0
	SENS2 = 1
	SENS3 = 2
	SENS4 = 3
	SENSor1 = 4
	SENSor2 = 5
	SENSor3 = 6
	SENSor4 = 7
	UNDefined = 8


# noinspection SpellCheckingInspection
class PulsMode(Enum):
	"""4 Members, DOUBle ... SINGle"""
	DOUBle = 0
	PHOPptrain = 1
	PTRain = 2
	SINGle = 3


# noinspection SpellCheckingInspection
class PulsTrigMode(Enum):
	"""3 Members, AUTO ... EXTernal"""
	AUTO = 0
	EGATe = 1
	EXTernal = 2


# noinspection SpellCheckingInspection
class RecScpiCmdMode(Enum):
	"""4 Members, AUTO ... OFF"""
	AUTO = 0
	DAUTo = 1
	MANual = 2
	OFF = 3


# noinspection SpellCheckingInspection
class RefLoOutput(Enum):
	"""3 Members, LO ... REF"""
	LO = 0
	OFF = 1
	REF = 2


# noinspection SpellCheckingInspection
class RoscBandWidtExt(Enum):
	"""2 Members, NARRow ... WIDE"""
	NARRow = 0
	WIDE = 1


# noinspection SpellCheckingInspection
class RoscFreqExt(Enum):
	"""4 Members, _1000MHZ ... _13MHZ"""
	_1000MHZ = 0
	_100MHZ = 1
	_10MHZ = 2
	_13MHZ = 3


# noinspection SpellCheckingInspection
class Rs232BdRate(Enum):
	"""7 Members, _115200 ... _9600"""
	_115200 = 0
	_19200 = 1
	_2400 = 2
	_38400 = 3
	_4800 = 4
	_57600 = 5
	_9600 = 6


# noinspection SpellCheckingInspection
class SatNavClockMode(Enum):
	"""2 Members, MSYMbol ... SYMBol"""
	MSYMbol = 0
	SYMBol = 1


# noinspection SpellCheckingInspection
class SlopeType(Enum):
	"""2 Members, NEGative ... POSitive"""
	NEGative = 0
	POSitive = 1


# noinspection SpellCheckingInspection
class SourceInt(Enum):
	"""2 Members, EXTernal ... INTernal"""
	EXTernal = 0
	INTernal = 1


# noinspection SpellCheckingInspection
class Test(Enum):
	"""4 Members, _0 ... STOPped"""
	_0 = 0
	_1 = 1
	RUNning = 2
	STOPped = 3


# noinspection SpellCheckingInspection
class TestExtIqMode(Enum):
	"""2 Members, IQIN ... IQOut"""
	IQIN = 0
	IQOut = 1


# noinspection SpellCheckingInspection
class TrigDelUnit(Enum):
	"""2 Members, SAMPle ... TIME"""
	SAMPle = 0
	TIME = 1


# noinspection SpellCheckingInspection
class TriggerSourceB(Enum):
	"""4 Members, BEXTernal ... OBASeband"""
	BEXTernal = 0
	EXTernal = 1
	INTernal = 2
	OBASeband = 3


# noinspection SpellCheckingInspection
class TrigMarkMode(Enum):
	"""6 Members, PATTern ... UNCHanged"""
	PATTern = 0
	PULSe = 1
	RATio = 2
	RESTart = 3
	TRIGger = 4
	UNCHanged = 5


# noinspection SpellCheckingInspection
class TrigRunMode(Enum):
	"""2 Members, RUN ... STOP"""
	RUN = 0
	STOP = 1


# noinspection SpellCheckingInspection
class UnitAngle(Enum):
	"""3 Members, DEGree ... RADian"""
	DEGree = 0
	DEGRee = 1
	RADian = 2


# noinspection SpellCheckingInspection
class UnitPower(Enum):
	"""3 Members, DBM ... V"""
	DBM = 0
	DBUV = 1
	V = 2


# noinspection SpellCheckingInspection
class UnitSlB(Enum):
	"""2 Members, SAMPle ... SEQuence"""
	SAMPle = 0
	SEQuence = 1


# noinspection SpellCheckingInspection
class UnitSlXmRadio(Enum):
	"""3 Members, MCM ... TPL"""
	MCM = 0
	SAMPle = 1
	TPL = 2


# noinspection SpellCheckingInspection
class Unknown(Enum):
	"""2 Members, DBM ... V"""
	DBM = 0
	V = 1


# noinspection SpellCheckingInspection
class UserPlug(Enum):
	"""18 Members, CIN ... TRIGger"""
	CIN = 0
	COUT = 1
	HIGH = 2
	LOW = 3
	MARRived = 4
	MKR1 = 5
	MKR2 = 6
	MLATency = 7
	NEXT = 8
	PEMSource = 9
	PETRigger = 10
	PVOut = 11
	SIN = 12
	SNValid = 13
	SOUT = 14
	SVALid = 15
	TOUT = 16
	TRIGger = 17


# noinspection SpellCheckingInspection
class XmRadioMarkMode(Enum):
	"""7 Members, MCM ... USER"""
	MCM = 0
	PATTern = 1
	PULSe = 2
	RATio = 3
	TPL = 4
	TRIGger = 5
	USER = 6


# noinspection SpellCheckingInspection
class XmRadioPhysLayer(Enum):
	"""6 Members, SAT1A ... TERRB"""
	SAT1A = 0
	SAT1B = 1
	SAT2A = 2
	SAT2B = 3
	TERRA = 4
	TERRB = 5



from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup
from ......Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class ClippingCls:
	"""Clipping commands group definition. 3 total commands, 0 Subgroups, 3 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("clipping", core, parent)

	def get_cfactor(self) -> float:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:CLIPping:CFACtor \n
		Snippet: value: float = driver.source.bb.arbitrary.mcarrier.clipping.get_cfactor() \n
		Sets the value of the desired crest factor, if baseband clipping is enabled (BB:ARB:MCAR:CLIP:STAT ON) . A Target Crest
		Factor above the crest factor of the unclipped multicarrier signal has no effect. \n
			:return: cfactor: float Range: -50 to 50, Unit: dB
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:MCARrier:CLIPping:CFACtor?')
		return Conversions.str_to_float(response)

	def set_cfactor(self, cfactor: float) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:CLIPping:CFACtor \n
		Snippet: driver.source.bb.arbitrary.mcarrier.clipping.set_cfactor(cfactor = 1.0) \n
		Sets the value of the desired crest factor, if baseband clipping is enabled (BB:ARB:MCAR:CLIP:STAT ON) . A Target Crest
		Factor above the crest factor of the unclipped multicarrier signal has no effect. \n
			:param cfactor: float Range: -50 to 50, Unit: dB
		"""
		param = Conversions.decimal_value_to_str(cfactor)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:MCARrier:CLIPping:CFACtor {param}')

	def get_cutoff(self) -> float:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:CLIPping:CUToff \n
		Snippet: value: float = driver.source.bb.arbitrary.mcarrier.clipping.get_cutoff() \n
		Sets the cut off frequency of the final lowpass filter, if baseband clipping is enabled (BB:ARB:MCAR:CLIP:STAT ON) . When
		the cut off frequency is set as half of the output sample rate, a final lowpass filter improves the spectrum of the
		clipped multicarrier signal, but may also increase the resulting crest factor. \n
			:return: cutoff: float Range: 0 to 120E6, Unit: MHz
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:MCARrier:CLIPping:CUToff?')
		return Conversions.str_to_float(response)

	def set_cutoff(self, cutoff: float) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:CLIPping:CUToff \n
		Snippet: driver.source.bb.arbitrary.mcarrier.clipping.set_cutoff(cutoff = 1.0) \n
		Sets the cut off frequency of the final lowpass filter, if baseband clipping is enabled (BB:ARB:MCAR:CLIP:STAT ON) . When
		the cut off frequency is set as half of the output sample rate, a final lowpass filter improves the spectrum of the
		clipped multicarrier signal, but may also increase the resulting crest factor. \n
			:param cutoff: float Range: 0 to 120E6, Unit: MHz
		"""
		param = Conversions.decimal_value_to_str(cutoff)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:MCARrier:CLIPping:CUToff {param}')

	def get_state(self) -> bool:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:CLIPping:[STATe] \n
		Snippet: value: bool = driver.source.bb.arbitrary.mcarrier.clipping.get_state() \n
		Switches baseband clipping on and off. Clipping reduces the peak power of the resulting multi carrier signal according to
		the value set with the command BB:ARB:MCAR:CLIP:CFAC. The resulting clipped peak power is defined by sum of the the RMS
		level of the unclipped multi carrier signal and the input parameter Target Crest Factor. Note that clipping reduces also
		the RMS level. Hence the resulting crest factor of the clipped signal is slightly above the Target Crest Factor. In order
		to get the unclipped parts of the clipped multicarrier signal matched with the unclipped multicarrier signal, the RF
		output power should be reduced by the difference between resulting crest factor and Target Crest Factor. \n
			:return: state: 0| 1| OFF| ON
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:MCARrier:CLIPping:STATe?')
		return Conversions.str_to_bool(response)

	def set_state(self, state: bool) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:CLIPping:[STATe] \n
		Snippet: driver.source.bb.arbitrary.mcarrier.clipping.set_state(state = False) \n
		Switches baseband clipping on and off. Clipping reduces the peak power of the resulting multi carrier signal according to
		the value set with the command BB:ARB:MCAR:CLIP:CFAC. The resulting clipped peak power is defined by sum of the the RMS
		level of the unclipped multi carrier signal and the input parameter Target Crest Factor. Note that clipping reduces also
		the RMS level. Hence the resulting crest factor of the clipped signal is slightly above the Target Crest Factor. In order
		to get the unclipped parts of the clipped multicarrier signal matched with the unclipped multicarrier signal, the RF
		output power should be reduced by the difference between resulting crest factor and Target Crest Factor. \n
			:param state: 0| 1| OFF| ON
		"""
		param = Conversions.bool_to_str(state)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:MCARrier:CLIPping:STATe {param}')



from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup
from ......Internal import Conversions
from ...... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class ExternalCls:
	"""External commands group definition. 3 total commands, 1 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("external", core, parent)

	@property
	def gate(self):
		"""gate commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_gate'):
			from .Gate import GateCls
			self._gate = GateCls(self._core, self._cmd_group)
		return self._gate

	# noinspection PyTypeChecker
	def get_impedance(self) -> enums.InputImpRf:
		"""SCPI: [SOURce<HW>]:PULM:TRIGger:EXTernal:IMPedance \n
		Snippet: value: enums.InputImpRf = driver.source.pulm.trigger.external.get_impedance() \n
		Selects the impedance for external pulse trigger. \n
			:return: impedance: G50| G10K
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:PULM:TRIGger:EXTernal:IMPedance?')
		return Conversions.str_to_scalar_enum(response, enums.InputImpRf)

	def set_impedance(self, impedance: enums.InputImpRf) -> None:
		"""SCPI: [SOURce<HW>]:PULM:TRIGger:EXTernal:IMPedance \n
		Snippet: driver.source.pulm.trigger.external.set_impedance(impedance = enums.InputImpRf.G10K) \n
		Selects the impedance for external pulse trigger. \n
			:param impedance: G50| G10K
		"""
		param = Conversions.enum_scalar_to_str(impedance, enums.InputImpRf)
		self._core.io.write(f'SOURce<HwInstance>:PULM:TRIGger:EXTernal:IMPedance {param}')

	# noinspection PyTypeChecker
	def get_slope(self) -> enums.SlopeType:
		"""SCPI: [SOURce<HW>]:PULM:TRIGger:EXTernal:SLOPe \n
		Snippet: value: enums.SlopeType = driver.source.pulm.trigger.external.get_slope() \n
		No command help available \n
			:return: slope: NEGative| POSitive
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:PULM:TRIGger:EXTernal:SLOPe?')
		return Conversions.str_to_scalar_enum(response, enums.SlopeType)

	def set_slope(self, slope: enums.SlopeType) -> None:
		"""SCPI: [SOURce<HW>]:PULM:TRIGger:EXTernal:SLOPe \n
		Snippet: driver.source.pulm.trigger.external.set_slope(slope = enums.SlopeType.NEGative) \n
		No command help available \n
			:param slope: NEGative| POSitive
		"""
		param = Conversions.enum_scalar_to_str(slope, enums.SlopeType)
		self._core.io.write(f'SOURce<HwInstance>:PULM:TRIGger:EXTernal:SLOPe {param}')

	def clone(self) -> 'ExternalCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = ExternalCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from typing import List

from .......Internal.Core import Core
from .......Internal.CommandsGroup import CommandsGroup
from .......Internal import Conversions
from .......Internal.Types import DataType
from .......Internal.StructBase import StructBase
from .......Internal.ArgStruct import ArgStruct
from .......Internal.ArgSingleList import ArgSingleList
from .......Internal.ArgSingle import ArgSingle


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class CoefficientsCls:
	"""Coefficients commands group definition. 4 total commands, 0 Subgroups, 4 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("coefficients", core, parent)

	def set(self, ipart_0: List[float], j_0: float, i_1: float, j_1: float) -> None:
		"""SCPI: [SOURce<HW>]:IQ:DPD:SHAPing:POLYnomial:COEFficients \n
		Snippet: driver.source.iq.dpd.shaping.polynomial.coefficients.set(ipart_0 = [1.1, 2.2, 3.3], j_0 = 1.0, i_1 = 1.0, j_1 = 1.0) \n
		Sets the polynomial coefficients as a list of up to 10 comma separated value pairs. In Cartesian coordinates system, the
		coefficients bn are expressed in degrees. \n
			:param ipart_0: No help available
			:param j_0: float Range: -1E6 to 1E6
			:param i_1: No help available
			:param j_1: float Range: -1E6 to 1E6
		"""
		param = ArgSingleList().compose_cmd_string(ArgSingle.as_open_list('ipart_0', ipart_0, DataType.FloatList, None), ArgSingle('j_0', j_0, DataType.Float), ArgSingle('i_1', i_1, DataType.Float), ArgSingle('j_1', j_1, DataType.Float))
		self._core.io.write(f'SOURce<HwInstance>:IQ:DPD:SHAPing:POLYnomial:COEFficients {param}'.rstrip())

	# noinspection PyTypeChecker
	class CoefficientsStruct(StructBase):
		"""Response structure. Fields: \n
			- Ipart_0: List[float]: No parameter help available
			- J_0: float: float Range: -1E6 to 1E6
			- I_1: float: No parameter help available
			- J_1: float: float Range: -1E6 to 1E6"""
		__meta_args_list = [
			ArgStruct('Ipart_0', DataType.FloatList, None, False, True, 1),
			ArgStruct.scalar_float('J_0'),
			ArgStruct.scalar_float('I_1'),
			ArgStruct.scalar_float('J_1')]

		def __init__(self):
			StructBase.__init__(self, self)
			self.Ipart_0: List[float] = None
			self.J_0: float = None
			self.I_1: float = None
			self.J_1: float = None

	def get(self) -> CoefficientsStruct:
		"""SCPI: [SOURce<HW>]:IQ:DPD:SHAPing:POLYnomial:COEFficients \n
		Snippet: value: CoefficientsStruct = driver.source.iq.dpd.shaping.polynomial.coefficients.get() \n
		Sets the polynomial coefficients as a list of up to 10 comma separated value pairs. In Cartesian coordinates system, the
		coefficients bn are expressed in degrees. \n
			:return: structure: for return value, see the help for CoefficientsStruct structure arguments."""
		return self._core.io.query_struct(f'SOURce<HwInstance>:IQ:DPD:SHAPing:POLYnomial:COEFficients?', self.__class__.CoefficientsStruct())

	def get_catalog(self) -> List[str]:
		"""SCPI: [SOURce<HW>]:IQ:DPD:SHAPing:POLYnomial:COEFficients:CATalog \n
		Snippet: value: List[str] = driver.source.iq.dpd.shaping.polynomial.coefficients.get_catalog() \n
		Queries the available polynomial files in the default directory. Only files with the file extension *.dpd_poly are listed. \n
			:return: catalog: string
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:DPD:SHAPing:POLYnomial:COEFficients:CATalog?')
		return Conversions.str_to_str_list(response)

	def load(self, filename: str) -> None:
		"""SCPI: [SOURce<HW>]:IQ:DPD:SHAPing:POLYnomial:COEFficients:LOAD \n
		Snippet: driver.source.iq.dpd.shaping.polynomial.coefficients.load(filename = '1') \n
		Loads the selected polynomial file. \n
			:param filename: string
		"""
		param = Conversions.value_to_quoted_str(filename)
		self._core.io.write(f'SOURce<HwInstance>:IQ:DPD:SHAPing:POLYnomial:COEFficients:LOAD {param}')

	def set_store(self, filename: str) -> None:
		"""SCPI: [SOURce<HW>]:IQ:DPD:SHAPing:POLYnomial:COEFficients:STORe \n
		Snippet: driver.source.iq.dpd.shaping.polynomial.coefficients.set_store(filename = '1') \n
		Saves the polynomial function as polynomial file. \n
			:param filename: string
		"""
		param = Conversions.value_to_quoted_str(filename)
		self._core.io.write(f'SOURce<HwInstance>:IQ:DPD:SHAPing:POLYnomial:COEFficients:STORe {param}')



from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup
from ......Internal import Conversions
from ......Internal.Types import DataType
from ......Internal.ArgSingleList import ArgSingleList
from ......Internal.ArgSingle import ArgSingle


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class StateCls:
	"""State commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("state", core, parent)

	def set(self, sec_pass_word: str, ssh_state: bool) -> None:
		"""SCPI: SYSTem:SECurity:NETWork:SSH:[STATe] \n
		Snippet: driver.system.security.network.ssh.state.set(sec_pass_word = '1', ssh_state = False) \n
		No command help available \n
			:param sec_pass_word: No help available
			:param ssh_state: No help available
		"""
		param = ArgSingleList().compose_cmd_string(ArgSingle('sec_pass_word', sec_pass_word, DataType.String), ArgSingle('ssh_state', ssh_state, DataType.Boolean))
		self._core.io.write(f'SYSTem:SECurity:NETWork:SSH:STATe {param}'.rstrip())

	def get(self) -> bool:
		"""SCPI: SYSTem:SECurity:NETWork:SSH:[STATe] \n
		Snippet: value: bool = driver.system.security.network.ssh.state.get() \n
		No command help available \n
			:return: ssh_state: No help available"""
		response = self._core.io.query_str(f'SYSTem:SECurity:NETWork:SSH:STATe?')
		return Conversions.str_to_bool(response)



from ........Internal.Core import Core
from ........Internal.CommandsGroup import CommandsGroup
from ........Internal.Types import DataType
from ........Internal.ArgSingleList import ArgSingleList
from ........Internal.ArgSingle import ArgSingle


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class AppendCls:
	"""Append commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("append", core, parent)

	def set(self, samp_count: float, frequency: float) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WSEGment:CONFigure:BLANk:APPend \n
		Snippet: driver.source.bb.arbitrary.wsegment.configure.blank.append.set(samp_count = 1.0, frequency = 1.0) \n
		Adds a blank segment to the multi segment file. \n
			:param samp_count: float Specifies the number of samples. Range: 512 to 1E7
			:param frequency: float Determines the clock rate. Range: 400 Hz to Max (depends on instrument type)
		"""
		param = ArgSingleList().compose_cmd_string(ArgSingle('samp_count', samp_count, DataType.Float), ArgSingle('frequency', frequency, DataType.Float))
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:WSEGment:CONFigure:BLANk:APPend {param}'.rstrip())



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal.Types import DataType
from ....Internal.ArgSingleList import ArgSingleList
from ....Internal.ArgSingle import ArgSingle


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class StateCls:
	"""State commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("state", core, parent)

	def set(self, data_set: int, source_file: str) -> None:
		"""SCPI: MMEMory:LOAD:STATe \n
		Snippet: driver.massMemory.load.state.set(data_set = 1, source_file = '1') \n
		Loads the specified file stored under the specified name in an internal memory. After the file has been loaded, the
		instrument setting must be activated using an *RCL command. \n
			:param data_set: No help available
			:param source_file: No help available
		"""
		param = ArgSingleList().compose_cmd_string(ArgSingle('data_set', data_set, DataType.Integer), ArgSingle('source_file', source_file, DataType.String))
		self._core.io.write(f'MMEMory:LOAD:STATe {param}'.rstrip())



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions
from .... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class ExternalCls:
	"""External commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("external", core, parent)

	# noinspection PyTypeChecker
	def get_frequency(self) -> enums.RoscFreqExt:
		"""SCPI: [SOURce<HW>]:ROSCillator:EXTernal:FREQuency \n
		Snippet: value: enums.RoscFreqExt = driver.source.roscillator.external.get_frequency() \n
		Selects the frequency of the external reference. \n
			:return: ext_freq: 10MHZ| 100MHZ| 1000MHZ| 13MHZ 13MHZ requires RF board with part number 1419.5308.02. To find out the RF board installed in the instrument: Select 'SGMA-GUI instrument name Setup Hardware Config' 'RF Assembly' Observe the part number of the assembly 'RfBoard'.
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:ROSCillator:EXTernal:FREQuency?')
		return Conversions.str_to_scalar_enum(response, enums.RoscFreqExt)

	def set_frequency(self, ext_freq: enums.RoscFreqExt) -> None:
		"""SCPI: [SOURce<HW>]:ROSCillator:EXTernal:FREQuency \n
		Snippet: driver.source.roscillator.external.set_frequency(ext_freq = enums.RoscFreqExt._1000MHZ) \n
		Selects the frequency of the external reference. \n
			:param ext_freq: 10MHZ| 100MHZ| 1000MHZ| 13MHZ 13MHZ requires RF board with part number 1419.5308.02. To find out the RF board installed in the instrument: Select 'SGMA-GUI instrument name Setup Hardware Config' 'RF Assembly' Observe the part number of the assembly 'RfBoard'.
		"""
		param = Conversions.enum_scalar_to_str(ext_freq, enums.RoscFreqExt)
		self._core.io.write(f'SOURce<HwInstance>:ROSCillator:EXTernal:FREQuency {param}')

	# noinspection PyTypeChecker
	def get_sbandwidth(self) -> enums.RoscBandWidtExt:
		"""SCPI: [SOURce]:ROSCillator:EXTernal:SBANdwidth \n
		Snippet: value: enums.RoscBandWidtExt = driver.source.roscillator.external.get_sbandwidth() \n
		Sets the synchronization bandwidth for an external reference signal. \n
			:return: sbandwidth: WIDE| NARRow NARRow The synchronization bandwidth is. WIDE Synchronization bandwidth is.
		"""
		response = self._core.io.query_str('SOURce:ROSCillator:EXTernal:SBANdwidth?')
		return Conversions.str_to_scalar_enum(response, enums.RoscBandWidtExt)

	def set_sbandwidth(self, sbandwidth: enums.RoscBandWidtExt) -> None:
		"""SCPI: [SOURce]:ROSCillator:EXTernal:SBANdwidth \n
		Snippet: driver.source.roscillator.external.set_sbandwidth(sbandwidth = enums.RoscBandWidtExt.NARRow) \n
		Sets the synchronization bandwidth for an external reference signal. \n
			:param sbandwidth: WIDE| NARRow NARRow The synchronization bandwidth is. WIDE Synchronization bandwidth is.
		"""
		param = Conversions.enum_scalar_to_str(sbandwidth, enums.RoscBandWidtExt)
		self._core.io.write(f'SOURce:ROSCillator:EXTernal:SBANdwidth {param}')



"""Contains conversion functions for SCPI string -> parameter and vice versa."""

import math
import struct
import sys
from enum import Enum
from typing import List, Tuple
from .ScpiEnums import ScpiEnum, enum_spec_prefixes, enum_spec_strings
from datetime import datetime

from . import Utilities
from .InstrumentErrors import RsInstrException


class BinFloatFormat(Enum):
	"""Binary format of a float number."""
	Single_4bytes = 1
	Single_4bytes_swapped = 2
	Double_8bytes = 3
	Double_8bytes_swapped = 4


class BinIntFormat(Enum):
	"""Binary format of an integer number."""
	Integer32_4bytes = 1
	Integer32_4bytes_swapped = 2
	Integer16_2bytes = 3
	Integer16_2bytes_swapped = 4


def assert_string_data(value: str) -> None:
	"""Asserts value is string type."""
	assert isinstance(value, str), f"Input value type must be string. Actual type: {type(value)}, value: {value}"


def assert_list_data(value: List) -> None:
	"""Asserts value is list type."""
	assert isinstance(value, list), f"Input value type must be a list. Actual type: {type(value)}, value: {value}"


def _get_endianness_symbol(swap_endianness: bool) -> str:
	"""Based on the current endianness returns the symbol used in the 'struct' module."""
	if swap_endianness is False:
		return '@'
	elif swap_endianness is True and sys.byteorder == 'little':
		return '>'
	else:
		return '<'


def bytes_to_float32_list(data: bytes, swap_endianness=False) -> List[float]:
	"""Converts bytes to list of floats - one number is represented by 4 bytes."""
	fmt = f'{_get_endianness_symbol(swap_endianness)}{len(data) // 4}f'
	return list(struct.unpack(fmt, data))


def bytes_to_double64_list(data: bytes, swap_endianness=False) -> List[float]:
	"""Converts bytes to list of doubles - one number is represented by 8 bytes."""
	fmt = f'{_get_endianness_symbol(swap_endianness)}{len(data) // 8}d'
	return list(struct.unpack(fmt, data))


def bytes_to_int32_list(data: bytes, swap_endianness=False) -> List[int]:
	"""Converts bytes to list of integer32 - one number is represented by 4 bytes."""
	fmt = f'{_get_endianness_symbol(swap_endianness)}{len(data) // 4}i'
	return list(struct.unpack(fmt, data))


def bytes_to_int16_list(data: bytes, swap_endianness=False) -> List[int]:
	"""Converts bytes to list of integer16 - one number is represented by 2 bytes."""
	fmt = f'{_get_endianness_symbol(swap_endianness)}{len(data) // 2}h'
	return list(struct.unpack(fmt, data))


def bytes_to_list_of_floats(data: bytes, fmt: BinFloatFormat) -> List[float]:
	"""Decodes binary data to a list of floating-point numbers based on the entered format."""
	if fmt == BinFloatFormat.Single_4bytes:
		return bytes_to_float32_list(data)
	elif fmt == BinFloatFormat.Single_4bytes_swapped:
		return bytes_to_float32_list(data, True)
	elif fmt == BinFloatFormat.Double_8bytes:
		return bytes_to_double64_list(data)
	elif fmt == BinFloatFormat.Double_8bytes_swapped:
		return bytes_to_double64_list(data, True)


def bytes_to_list_of_integers(data: bytes, fmt: BinIntFormat) -> List[int]:
	"""Decodes binary data to a list of integer numbers based on the entered format."""
	if fmt == BinIntFormat.Integer32_4bytes:
		return bytes_to_int32_list(data)
	elif fmt == BinIntFormat.Integer32_4bytes_swapped:
		return bytes_to_int32_list(data, True)
	elif fmt == BinIntFormat.Integer16_2bytes:
		return bytes_to_int16_list(data)
	elif fmt == BinIntFormat.Integer16_2bytes_swapped:
		return bytes_to_int16_list(data, True)


def double64_list_to_bytes(data: List[float], swap_endianness=False) -> bytes:
	"""Converts list of doubles to bytes - one number is converted to 8 bytes."""
	fmt = f'{_get_endianness_symbol(swap_endianness)}{str(len(data))}d'
	return struct.pack(fmt, *data)


def float32_list_to_bytes(data: List[float], swap_endianness=False) -> bytes:
	"""Converts list of floats to bytes - one number is converted to 4 bytes."""
	fmt = f'{_get_endianness_symbol(swap_endianness)}{str(len(data))}f'
	return struct.pack(fmt, *data)


def int32_list_to_bytes(data: List[int], swap_endianness=False) -> bytes:
	"""Converts list of integers 32 to bytes - one number is converted to 4 bytes."""
	fmt = f'{_get_endianness_symbol(swap_endianness)}{str(len(data))}i'
	return struct.pack(fmt, *data)


def int16_list_to_bytes(data: List[float], swap_endianness=False) -> bytes:
	"""Converts list of integers 16 to bytes - one number is converted to 2 bytes."""
	fmt = f'{_get_endianness_symbol(swap_endianness)}{str(len(data))}h'
	return struct.pack(fmt, *data)


def list_of_integers_to_bytes(ints: List[int], fmt: BinIntFormat) -> bytes:
	"""Encodes list of integers to binary data based on the entered format."""
	if fmt == BinIntFormat.Integer32_4bytes:
		return int32_list_to_bytes(ints)
	elif fmt == BinIntFormat.Integer32_4bytes_swapped:
		return int32_list_to_bytes(ints, True)
	elif fmt == BinIntFormat.Integer16_2bytes:
		return int16_list_to_bytes(ints)
	elif fmt == BinIntFormat.Integer16_2bytes_swapped:
		return int16_list_to_bytes(ints, True)


def list_of_floats_to_bytes(floats: List[float], fmt: BinFloatFormat) -> bytes:
	"""Encodes list of floats to binary data based on the entered format."""
	if fmt == BinFloatFormat.Single_4bytes:
		return float32_list_to_bytes(floats)
	elif fmt == BinFloatFormat.Single_4bytes_swapped:
		return float32_list_to_bytes(floats, True)
	elif fmt == BinFloatFormat.Double_8bytes:
		return double64_list_to_bytes(floats)
	elif fmt == BinFloatFormat.Double_8bytes_swapped:
		return double64_list_to_bytes(floats, True)


pure_bool_true_lookup = frozenset(['on', 'On', 'ON', 'true', 'True', 'TRUE'])
bool_true_lookup = frozenset(['1', 'on', 'On', 'ON', 'true', 'True', 'TRUE'])
bool_false_lookup = frozenset(['0', 'off', 'Off', 'OFF', 'false', 'False', 'FALSE'])
pure_bool_false_lookup = frozenset(['off', 'Off', 'OFF', 'false', 'False', 'FALSE'])


def str_to_bool(string: str) -> bool:
	"""Converts string to boolean value.
	The function is robust, and case-insensitive.
	If the string can not be converted to a boolean, the function returns False."""
	assert_string_data(string)
	if string in bool_true_lookup:
		return True
	if string in bool_false_lookup:
		return False
	# If leading/trailing spaces
	string = string.strip()
	if string in bool_true_lookup:
		return True
	if string in bool_false_lookup:
		return False
	# If enclosed by brackets
	string = Utilities.trim_str_response(string)
	if string in bool_true_lookup:
		return True
	if string in bool_false_lookup:
		return False
	return False


def string_to_pure_bool(string: str) -> bool or None:
	"""Converts string to boolean value. Compare to str_to_bool(), the values '1' and '0' are not considered boolean.
	Also, if the method can not convert the string to boolean, it returns None."""
	assert_string_data(string)
	if string in pure_bool_true_lookup:
		return True
	if string in pure_bool_false_lookup:
		return False
	# If leading/trailing spaces
	string = string.strip()
	if string in pure_bool_true_lookup:
		return True
	if string in pure_bool_false_lookup:
		return False
	# If enclosed by brackets
	string = Utilities.trim_str_response(string)
	if string in pure_bool_true_lookup:
		return True
	if string in pure_bool_false_lookup:
		return False
	return None


number_plus_inf_lookup = frozenset(['Inf', 'INF', 'INFINITY', '+Inf', '+INF', '+inf', '+INFINITY', '+Infinity', '+infinity'])
number_minus_inf_lookup = frozenset(['-Inf', '-INF', '-inf', '-INFINITY', '-Infinity', '-infinity'])
number_nan_lookup = frozenset(['Nan', 'NAN', 'nan', 'NaN', 'NAV', 'NaV', 'NCAP', 'INV', 'NONE', 'none', 'None', 'DTX', 'UND', 'und'])
number_max_lookup = frozenset(['OFL', 'ofl', 'Ofl'])
number_min_lookup = frozenset(['UFL', 'ufl', 'Ufl'])
number_si_suffix = {'pHz': 1E-12, 'MHz': 1E+6, 'kHz': 1E+3, 'GHz': 1E+9, 'mHz': 1E-3, 'uHz': 1E-6, 'µHz': 1E-6, 'THz': 1E+12, 'nHz': 1E-9, 'ns': 1E-9, 'fW': 1E-15,
					'pW': 1E-12, 'nW': 1E-9, 'uW': 1E-6, 'µW': 1E-6, 'mW': 1E-3, 'kW': 1E3, 'MW': 1E6, 'GW': 1E9, 'MV': 1E+6, 'MA': 1E+6, 'ps': 1E-12, 'fs': 1E-15,
					'km': 1E+3, 'kV': 1E+3, 'kA': 1E+3, 'pF': 1E-2, 'Hz': 1.0, 'mm': 1E-3, 'mA': 1E-3, 'mF': 1E-3, 'mV': 1E-3, 'pV': 1E-12, 'nF': 1E-9, 'nA': 1E-9,
					'nV': 1E-9, 'nm': 1E-9, 'pm': 1E-12, 'us': 1E-6, 'µs': 1E-6, 'uF': 1E-6, 'µF': 1E-6, 'ms': 1E-3, 'uA': 1E-6, 'µA': 1E-6, 'uV': 1E-6, 'µV': 1E-6,
					'um': 1E-6, 'µm': 1E-6, 'pA': 1E-12, 'V': 1, 'W': 1, 'A': 1, 'F': 1, 's': 1, 'm': 1}
int_neg_inf = -(sys.maxsize - 1)


def strip_si_suffix(string: str) -> Tuple[bool, str, float]:
	"""Tries to find defined suffixes in the text and returns the stripped text and the multiplier as double number.
	If no known suffix is detected, the method returns false, strippedText=text, multiplier=1.0
	Example: text='123 MHz' strippedText='123' multiplier=1E6"""
	for suffix in number_si_suffix.keys():
		if string.endswith(suffix):
			return True, string[:-len(suffix)].rstrip(), number_si_suffix[suffix]
	return False, string, 1.0


def str_to_int(string: str) -> int:
	"""Converts string to integer value. Float values are coerced to integer.
	Also recognizes case-insensitive special values like NaN, INV, NCAP..."""
	assert_string_data(string)
	string = string.strip(" \t\r\n'\"")
	if string == '':
		return 0
	value = str_special_values_to_int(string)
	if value:
		return value

	# Hexadecimal numbers
	if string.startswith('#H') or string.startswith('0x'):
		if ',' in string:
			return int(string[2:string.find(',')], 16)
		else:
			return int(string[2:], 16)

	# Binary numbers
	if string.startswith('#B') or string.startswith('0b'):
		if ',' in string:
			return int(string[2:string.find(',')], 2)
		else:
			return int(string[2:], 2)

	# Octal numbers
	if string.startswith('#Q') or string.startswith('0o'):
		if ',' in string:
			return int(string[2:string.find(',')], 8)
		else:
			return int(string[2:], 8)
	# Simulation
	if string == 'Simulating':
		return 0
	try:
		return int(round(float(string)))
	except ValueError:
		result = strip_si_suffix(string)
		if result[0] is False:
			raise
		try:
			return int(round(float(result[1]) * result[2]))
		except ValueError:
			raise ValueError(f"could not convert string to integer: '{string}'")


def str_special_values_to_int(string: str) -> int:
	"""Converts special string values to integer. Returns None if no special value was found."""
	assert_string_data(string)
	if string in number_plus_inf_lookup or string in number_max_lookup:
		return sys.maxsize
	if string in number_minus_inf_lookup or string in number_min_lookup or string in number_nan_lookup:
		return int_neg_inf
	if string == 'OFF':
		return int_neg_inf + 1
	if string == 'ON':
		return int_neg_inf + 2
	if string == 'OK':
		return sys.maxsize - 1
	if string == 'DC':
		# noinspection PyTypeChecker
		return int_neg_inf / 100
	if string == 'ULEU':
		return int(sys.maxsize / 10)
	if string == 'ULEL':
		# noinspection PyTypeChecker
		return int_neg_inf / 10
	# noinspection PyTypeChecker
	return None


def str_to_int_or_bool(string: str) -> int or bool:
	"""Similar to str_to_int, but for special values "ON/OFF" the function returns boolean"""
	result = string_to_pure_bool(string)
	if result is not None:
		return result
	return str_to_int(string)


def str_to_float(string: str) -> float:
	"""Converts string to float value.
	Also recognizes case-insensitive special values like NaN, INV, NCAP..."""
	assert_string_data(string)
	string = string.strip(" \t\r\n'\"")
	if string == '':
		return 0.0
	if string in number_plus_inf_lookup:
		return math.inf
	if string in number_minus_inf_lookup:
		return -math.inf
	if string in number_nan_lookup:
		return math.nan
	if string in number_max_lookup:
		return sys.float_info.max
	if string in number_min_lookup:
		return -sys.float_info.max
	if string == 'OFF':
		return -sys.float_info.epsilon
	if string == 'ON':
		return -2*sys.float_info.epsilon
	if string == 'OK':
		return sys.float_info.epsilon
	if string == 'DC' or string == '':
		return -sys.float_info.max / 100
	if string == 'ULEU':
		return sys.float_info.max / 10
	if string == 'ULEL':
		return -sys.float_info.max / 10
	if string == 'Simulating':
		return 0.0
	try:
		return float(string)
	except ValueError:
		result = strip_si_suffix(string)
		if result[0] is False:
			raise
		try:
			return float(result[1]) * result[2]
		except ValueError:
			raise ValueError(f"could not convert string to float: '{string}'")


def str_to_float_or_bool(string: str) -> float or bool:
	"""Similar to str_to_float, but for special values "ON/OFF" the function returns boolean"""
	result = string_to_pure_bool(string)
	if result is not None:
		return result
	return str_to_float(string)


def float_to_str(value: float) -> str:
	"""Converts double number to string using {.12g} formatter."""
	return format(value, ".12g")


def bool_to_str(value: bool) -> str:
	"""Converts boolean to 'ON' or 'OFF' string."""
	if type(value) is bool:
		return 'ON' if value is True else 'OFF'
	else:
		raise RsInstrException(f"bool_to_str: unsupported variable type '{type(value)}', value '{value}'. Only boolean values are supported.")


def str_enclose_by_quotes(string: str) -> str:
	"""Returns string enclosed by single quotes."""
	assert_string_data(string)
	return "'" + string + "'"


def list_to_csv_str(value: List, delimiter: str = ',') -> str:
	"""Converts list of elements to strings separated by commas.
	Element types can differ on an individual basis.
	Supported element types:
	- int
	- bool
	- float
	- string -> string no quotes
	- enum"""
	assert_list_data(value)
	result = []
	for x in value:
		el = value_to_str(x)
		if not el:
			raise TypeError(f"List element type is not supported by Conversions.list_to_csv_str: '{x}'")
		result.append(el)
	return delimiter.join(result)


def list_to_csv_quoted_str(value: List) -> str:
	"""Converts list of elements to quoted strings separated by commas.
	Only string elements are enclosed by single quotes
	Element types can differ on an individual basis.
	Supported element types:
	- int
	- bool
	- float
	- string -> string enclosed by quotes
	- enum"""
	assert_list_data(value)
	result = []
	for x in value:
		if isinstance(x, str):
			el = str_enclose_by_quotes(x)
		else:
			el = value_to_str(x)
		if not el:
			raise TypeError(f"List element type is not supported by Conversions.list_to_csv_quoted_str: '{x}'")
		result.append(el)
	return ','.join(result)


def decimal_value_to_str(x: int or float) -> str:
	"""Converts scalar decimal value to string.
	Supported element types:
	- int
	- float"""
	if isinstance(x, int) and type(x) is not bool:
		return str(x)
	elif isinstance(x, float):
		return float_to_str(x)
	else:
		raise RsInstrException(f"decimal_value_to_str: unsupported variable type '{type(x)}', value '{x}'. Only integer and float types are supported.")


def decimal_or_bool_value_to_str(x: int or float or bool) -> str:
	"""Converts scalar decimal value to string.
	Supported element types:
	- int
	- float
	- boolean"""
	if type(x) is bool:
		return bool_to_str(x)
	if isinstance(x, int):
		return str(x)
	elif isinstance(x, float):
		return float_to_str(x)
	else:
		raise RsInstrException(f"decimal_or_bool_value_to_str: unsupported variable type '{type(x)}', value '{x}'. Only integer, float and boolean types are supported.")


def value_to_str(x: int or bool or float or str or Enum) -> str:
	"""Converts scalar value to string.
	Supported element types:
	- int
	- bool
	- float
	- string
	- enum"""
	if isinstance(x, bool):
		return bool_to_str(x)
	elif isinstance(x, int):
		return str(x)
	elif isinstance(x, float):
		return float_to_str(x)
	elif isinstance(x, str):
		return x
	elif isinstance(x, Enum):
		if isinstance(x.value, str):
			return enum_value_to_scpi_string(x.value)
		return enum_value_to_scpi_string(x.name)
	else:
		raise RsInstrException(f"value_to_str: unsupported variable type '{type(x)}', value '{x}'. Supported types: int, bool, float, str, enum.")


def enum_value_to_scpi_string(enum_value: str) -> str:
	"""Conversion EnumValue -> SCPI_String
	Unescapes all the special characters that can not be contained in the enum member definition, but can be sent to the instrument as enum string.
	Use this to send the scpi enum value to the instrument."""
	for key in enum_spec_prefixes:
		if enum_value.startswith(key):
			enum_value = enum_spec_prefixes[key] + enum_value[len(key):]
	for key in enum_spec_strings:
		enum_value = enum_value.replace(key, enum_spec_strings[key])
	return enum_value


def value_to_quoted_str(x: int or bool or float or str or Enum) -> str:
	"""Converts scalar value to string enclosed by single quotes.
	Supported element types:
	- int
	- bool
	- float
	- string
	- enum"""
	return f"'{value_to_str(x)}'"


def str_to_float_list(string: str) -> List[float]:
	"""Converts string with comma-separated values to list of Floats."""
	assert_string_data(string)
	if not string:
		return []
	result = [*map(str_to_float, string.split(','))]
	return result


def str_to_float_or_bool_list(string: str) -> List[float or bool]:
	"""Converts string with comma-separated values to list of float or boolean values."""
	assert_string_data(string)
	if not string:
		return []
	result = [*map(str_to_float_or_bool, string.split(','))]
	return result


def str_to_int_list(string: str) -> List[int]:
	"""Converts string with comma-separated values to list of Integers."""
	assert_string_data(string)
	if not string:
		return []
	result = [*map(str_to_int, string.split(','))]
	return result


def str_to_int_or_bool_list(string: str) -> List[int or bool]:
	"""Converts string with comma-separated values to list of integer or boolean values."""
	assert_string_data(string)
	if not string:
		return []
	result = [*map(str_to_int_or_bool, string.split(','))]
	return result


def str_to_bool_list(string: str) -> List[bool]:
	"""Converts string with comma-separated values to list of booleans."""
	assert_string_data(string)
	if not string:
		return []
	result = [*map(str_to_bool, string.split(','))]
	return result


def str_to_str_list(string: str, clear_one_empty_item: bool = False) -> List[str]:
	"""Converts string with comma-separated values to list of strings.
	Each element is trimmed by trim_str_response().
	If the clear_one_empty_item is set to True (default is False), and the result is exactly one empty string item, the method returns empty list."""
	assert_string_data(string)
	if not string:
		return []
	result = [*map(Utilities.trim_str_response, string.split(','))]
	if clear_one_empty_item and len(result) == 1 and result[0] == '':
		return []
	return result


def str_to_scalar_enum_helper(string: str, scpi_enum: ScpiEnum, array_search: bool, exc_if_not_found) -> Enum:
	"""Converts string to one enum element.
	array_search signal no need to force the comma removing,
	because the elements definitely do not have any commas - commas have been used to split string to the list of strings
	The function can also return:
	- integer special value, if the string was not found in the enum, and it is a special value.
	- input string, if the string was not found and raise_if_not_found is set to False - used for the EnumExt types."""
	if scpi_enum.has_quotes:
		value = Utilities.trim_str_response(string, mode=Utilities.TrimStringMode.white_chars_double_quotes)
	else:
		value = Utilities.trim_str_response(string)
	enum_value = scpi_enum.find_in_enum_members(value, False)
	if enum_value is not None:
		return enum_value
	if array_search is False:
		# If the result is still -1 (not found), try to force removing the comma in the string.
		enum_value = scpi_enum.find_in_enum_members(value, True)
		if enum_value is not None:
			return enum_value
	# If not found, search in the special integer numbers:
	spec_value = str_special_values_to_int(value)
	if spec_value:
		# noinspection PyTypeChecker
		return spec_value
	if exc_if_not_found:
		raise RsInstrException(f"String '{value}' can not be found in the enum type '{scpi_enum.enum_type}'")
	# noinspection PyTypeChecker
	return Utilities.trim_str_response(string)


def str_to_simple_scalar_enum(string: str, enum_type, case_sensitive: bool = True, ignore_underscores: bool = False) -> Enum or None:
	"""Converts string to one enum element.
	Does not handle special value or non-mandatory parts.
	Function is used in core only for standard enum conversions, not for SCPI enum conversions."""
	value = Utilities.trim_str_response(string)
	enum_members = [x.name for x in enum_type]
	enum_members_mod = [x.name for x in enum_type]
	if not case_sensitive:
		enum_members_mod = [x.upper() for x in enum_members]
		value = value.upper()
	if ignore_underscores:
		enum_members_mod = [x.replace('_', '') for x in enum_members_mod]
		value = value.replace('_', '')
	if value in enum_members_mod:
		return enum_type[enum_members[enum_members_mod.index(value)]]
	return None


def str_to_list_enum_helper(string: str, scpi_enum: ScpiEnum, exc_if_not_found: bool = True) -> List[Enum]:
	"""Converts string to list of enum elements. separated by comma"""
	elements = string.split(',')
	return [str_to_scalar_enum_helper(x, scpi_enum, True, exc_if_not_found) for x in elements]


def enum_scalar_to_str(data, enum_type) -> str:
	"""Converts enum scalar value to string."""
	assert isinstance(data, enum_type), f"Expected command parameter {enum_type}, actual data type: {type(data)}. Value: {data}"
	return value_to_str(data)


def enum_ext_scalar_to_str(data, enum_type) -> str:
	"""Converts enum scalar value to string.
	If the input value is string, the function returns the string with single quotes."""
	if isinstance(data, str):
		# Return string with quotes
		return value_to_quoted_str(Utilities.trim_str_response(data))
	assert isinstance(data, enum_type), f"Expected command parameter string or {enum_type}, actual data type: {type(data)}. Value: {data}"
	return value_to_str(data)


def enum_list_to_str(data: List, enum_type) -> str:
	"""Converts enum list to csv-string."""
	# For enums, check that each element is an enum
	assert all(isinstance(x, enum_type) for x in data), f"Expected command parameter list of {enum_type}, detected one or more elements of non-enum type. Value: {data}"
	return list_to_csv_str(data)


def enum_ext_list_to_str(data: List, enum_type) -> str:
	"""Converts enum list to csv-string. Allows the elements to be either enum or string."""
	assert all((isinstance(x, enum_type or str) or isinstance(x, str)) for x in data), f"Expected command parameter list of strings or {enum_type}, detected one or more elements of non-enum/non-string type. Value: {data}"
	return list_to_csv_quoted_str(data)


def str_to_scalar_enum(string: str, enum_type) -> Enum:
	"""Converts string to one enum element.
	Throws exception if the string can not be converted to an enum element or a special value."""
	return str_to_scalar_enum_helper(string, ScpiEnum(enum_type), False, exc_if_not_found=True)


def str_to_scalar_enum_ext(string: str, enum_type) -> Enum:
	"""Converts string to one enum element.
	Compared to str_to_scalar_enum, in case the string can not be converted, it is returned trimmed for quotes and ."""
	return str_to_scalar_enum_helper(string, ScpiEnum(enum_type), False, exc_if_not_found=False)


def str_to_list_enum(string: str, enum_type) -> List[Enum]:
	"""Converts string to list of enum elements."""
	return str_to_list_enum_helper(string, ScpiEnum(enum_type))


def str_to_list_enum_ext(string: str, enum_type) -> List[Enum]:
	"""Converts string to list of enum or string elements."""
	return str_to_list_enum_helper(string, ScpiEnum(enum_type), exc_if_not_found=False)


def convert_ts_to_datetime(timestamp: datetime or float) -> datetime:
	"""Converts timestamp as float to datetime. For datetime tuple it just passes the value."""
	if isinstance(timestamp, float) or isinstance(timestamp, int):
		return datetime.fromtimestamp(timestamp)
	return timestamp


def get_timestamp_string(timestamp: datetime or float) -> str:
	"""Returns the timestamp as string. The timestamp can be a datetime tuple or float seconds coming from the time.time()."""
	timestamp = convert_ts_to_datetime(timestamp)
	cur_time = timestamp.strftime('%H:%M:%S.%f')[:-3]
	return cur_time


def get_timedelta_fixed_string(time_start: datetime or float, time_end: datetime or float) -> str:
	"""Returns the time span as string - fixed in the format of '%H:%M:%S.%f'."""
	time_a = convert_ts_to_datetime(time_start)
	time_b = convert_ts_to_datetime(time_end)
	frac = (time_b - time_a).total_seconds()
	wh = math.floor(frac)
	d = int(wh / 86400)
	h = int((wh - (d * 86400)) / 3600)
	m = int((wh - (d * 86400 + h * 3600)) / 60)
	s = int((wh - (d * 86400 + h * 3600 + m * 60)))
	ms = int((frac - wh) * 1000)
	res = f'{h:02d}:{m:02d}:{s:02d}.{ms:03d}'
	if d > 0:
		res = f'{d}d ' + res
	return res


def get_timedelta_string(time_a: datetime or float, time_b: datetime or float) -> str:
	"""Returns the time span as string - dynamic based on the difference."""
	time_a = convert_ts_to_datetime(time_a)
	time_b = convert_ts_to_datetime(time_b)
	if time_b < time_a:
		return '0.000 ms'
	diff = time_b - time_a
	if diff.seconds < 10:
		return f'{diff.total_seconds() * 1000:0.3f} ms'
	elif diff.seconds < 1000:
		a = diff.total_seconds()
		return f'{a:0.3f} secs'
	hours, remainder = divmod(diff.seconds, 3600)
	minutes, seconds = divmod(remainder, 60)
	return f'{hours:02d}:{minutes:02d}:{seconds:02d}'



from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup
from ...Internal.Utilities import trim_str_response


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class QueueCls:
	"""Queue commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("queue", core, parent)

	def get_next(self) -> str:
		"""SCPI: STATus:QUEue:[NEXT] \n
		Snippet: value: str = driver.status.queue.get_next() \n
		Queries the oldest entry in the error queue and then deletes it. Positive error numbers denote device-specific errors,
		and negative error numbers denote error messages defined by SCPI. If the error queue is empty, 0 ('No error') is returned.
		The command is identical to SYSTem:ERRor[:NEXT]?. \n
			:return: next_py: string
		"""
		response = self._core.io.query_str('STATus:QUEue:NEXT?')
		return trim_str_response(response)



from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup
from ......Internal import Conversions
from ...... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class GateCls:
	"""Gate commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("gate", core, parent)

	# noinspection PyTypeChecker
	def get_polarity(self) -> enums.NormalInverted:
		"""SCPI: [SOURce<HW>]:PULM:TRIGger:EXTernal:GATE:POLarity \n
		Snippet: value: enums.NormalInverted = driver.source.pulm.trigger.external.gate.get_polarity() \n
		Selects the polarity of the Gate signal. \n
			:return: polarity: NORMal| INVerted
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:PULM:TRIGger:EXTernal:GATE:POLarity?')
		return Conversions.str_to_scalar_enum(response, enums.NormalInverted)

	def set_polarity(self, polarity: enums.NormalInverted) -> None:
		"""SCPI: [SOURce<HW>]:PULM:TRIGger:EXTernal:GATE:POLarity \n
		Snippet: driver.source.pulm.trigger.external.gate.set_polarity(polarity = enums.NormalInverted.INVerted) \n
		Selects the polarity of the Gate signal. \n
			:param polarity: NORMal| INVerted
		"""
		param = Conversions.enum_scalar_to_str(polarity, enums.NormalInverted)
		self._core.io.write(f'SOURce<HwInstance>:PULM:TRIGger:EXTernal:GATE:POLarity {param}')



from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup
from ......Internal import Conversions
from ...... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class ClockCls:
	"""Clock commands group definition. 6 total commands, 1 Subgroups, 4 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("clock", core, parent)

	@property
	def synchronization(self):
		"""synchronization commands group. 1 Sub-classes, 1 commands."""
		if not hasattr(self, '_synchronization'):
			from .Synchronization import SynchronizationCls
			self._synchronization = SynchronizationCls(self._core, self._cmd_group)
		return self._synchronization

	# noinspection PyTypeChecker
	def get_mode(self) -> enums.ArbClockMode:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:CLOCk:MODE \n
		Snippet: value: enums.ArbClockMode = driver.source.bb.arbitrary.clock.get_mode() \n
		The command enters the type of externally supplied clock (BB:ARB:CLOCk:SOURce EXTernal) . When MSAMple is used,
		a multiple of the sample clock is supplied via the CLOCK connector and the sample clock is derived internally from this.
		The multiplier is entered with the command BB:ARB:CLOCk:MULTiplier. \n
			:return: mode: SAMPle| MSAMple
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:CLOCk:MODE?')
		return Conversions.str_to_scalar_enum(response, enums.ArbClockMode)

	def set_mode(self, mode: enums.ArbClockMode) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:CLOCk:MODE \n
		Snippet: driver.source.bb.arbitrary.clock.set_mode(mode = enums.ArbClockMode.MSAMple) \n
		The command enters the type of externally supplied clock (BB:ARB:CLOCk:SOURce EXTernal) . When MSAMple is used,
		a multiple of the sample clock is supplied via the CLOCK connector and the sample clock is derived internally from this.
		The multiplier is entered with the command BB:ARB:CLOCk:MULTiplier. \n
			:param mode: SAMPle| MSAMple
		"""
		param = Conversions.enum_scalar_to_str(mode, enums.ArbClockMode)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:CLOCk:MODE {param}')

	def get_multiplier(self) -> int:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:CLOCk:MULTiplier \n
		Snippet: value: int = driver.source.bb.arbitrary.clock.get_multiplier() \n
		The command specifies the multiplier for clock type 'Multiple Samples' (BB:ARB:CLOCk:MODE MSAM) in the case of an
		external clock source. \n
			:return: multiplier: integer Range: 1 to 64
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:CLOCk:MULTiplier?')
		return Conversions.str_to_int(response)

	def set_multiplier(self, multiplier: int) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:CLOCk:MULTiplier \n
		Snippet: driver.source.bb.arbitrary.clock.set_multiplier(multiplier = 1) \n
		The command specifies the multiplier for clock type 'Multiple Samples' (BB:ARB:CLOCk:MODE MSAM) in the case of an
		external clock source. \n
			:param multiplier: integer Range: 1 to 64
		"""
		param = Conversions.decimal_value_to_str(multiplier)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:CLOCk:MULTiplier {param}')

	# noinspection PyTypeChecker
	def get_source(self) -> enums.ClocSourBb:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:CLOCk:SOURce \n
		Snippet: value: enums.ClocSourBb = driver.source.bb.arbitrary.clock.get_source() \n
		Sets the source for the digital modulation clock. \n
			:return: source: INTernal| EXTernal INTernal The internal clock reference is used. EXTernal The external clock reference is supplied to the connector.
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:CLOCk:SOURce?')
		return Conversions.str_to_scalar_enum(response, enums.ClocSourBb)

	def set_source(self, source: enums.ClocSourBb) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:CLOCk:SOURce \n
		Snippet: driver.source.bb.arbitrary.clock.set_source(source = enums.ClocSourBb.AINTernal) \n
		Sets the source for the digital modulation clock. \n
			:param source: INTernal| EXTernal INTernal The internal clock reference is used. EXTernal The external clock reference is supplied to the connector.
		"""
		param = Conversions.enum_scalar_to_str(source, enums.ClocSourBb)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:CLOCk:SOURce {param}')

	def get_value(self) -> float:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:CLOCk \n
		Snippet: value: float = driver.source.bb.arbitrary.clock.get_value() \n
		The command sets the clock rate in samples. Loading a waveform waveform (ARB:WAV:SEL <name>) sets the clock rate that is
		defined in the waveform tag 'clock'. The command subsequently changes the clock rate; see data sheet for value range. In
		the case of an external clock source (selection ARB:CLOCk:SOURce EXTernal) the clock for the external source must be
		entered with this command. \n
			:return: clock: float
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:CLOCk?')
		return Conversions.str_to_float(response)

	def set_value(self, clock: float) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:CLOCk \n
		Snippet: driver.source.bb.arbitrary.clock.set_value(clock = 1.0) \n
		The command sets the clock rate in samples. Loading a waveform waveform (ARB:WAV:SEL <name>) sets the clock rate that is
		defined in the waveform tag 'clock'. The command subsequently changes the clock rate; see data sheet for value range. In
		the case of an external clock source (selection ARB:CLOCk:SOURce EXTernal) the clock for the external source must be
		entered with this command. \n
			:param clock: float
		"""
		param = Conversions.decimal_value_to_str(clock)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:CLOCk {param}')

	def clone(self) -> 'ClockCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = ClockCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup
from ......Internal import Conversions
from ......Internal.Types import DataType
from ......Internal.ArgSingleList import ArgSingleList
from ......Internal.ArgSingle import ArgSingle
from ...... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class ValueCls:
	"""Value commands group definition. 3 total commands, 0 Subgroups, 3 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("value", core, parent)

	def get(self, xvalue: float, xunit: enums.Unknown) -> float:
		"""SCPI: [SOURce<HW>]:IQ:DOHerty:AMPM:VALue \n
		Snippet: value: float = driver.source.iq.doherty.amPm.value.get(xvalue = 1.0, xunit = enums.Unknown.DBM) \n
		No command help available \n
			:param xvalue: No help available
			:param xunit: No help available
			:return: delta_phase: No help available"""
		param = ArgSingleList().compose_cmd_string(ArgSingle('xvalue', xvalue, DataType.Float), ArgSingle('xunit', xunit, DataType.Enum, enums.Unknown))
		response = self._core.io.query_str(f'SOURce<HwInstance>:IQ:DOHerty:AMPM:VALue? {param}'.rstrip())
		return Conversions.str_to_float(response)

	def get_level(self) -> float:
		"""SCPI: [SOURce<HW>]:IQ:DOHerty:AMPM:VALue:LEVel \n
		Snippet: value: float = driver.source.iq.doherty.amPm.value.get_level() \n
		No command help available \n
			:return: delta_phase: No help available
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:DOHerty:AMPM:VALue:LEVel?')
		return Conversions.str_to_float(response)

	def get_pep(self) -> float:
		"""SCPI: [SOURce<HW>]:IQ:DOHerty:AMPM:VALue:PEP \n
		Snippet: value: float = driver.source.iq.doherty.amPm.value.get_pep() \n
		No command help available \n
			:return: delta_phase: No help available
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:DOHerty:AMPM:VALue:PEP?')
		return Conversions.str_to_float(response)



from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup
from ......Internal import Conversions
from ...... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class PowerCls:
	"""Power commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("power", core, parent)

	# noinspection PyTypeChecker
	def get_reference(self) -> enums.ArbMultCarrLevRef:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:POWer:REFerence \n
		Snippet: value: enums.ArbMultCarrLevRef = driver.source.bb.arbitrary.mcarrier.power.get_reference() \n
		Defines the way the individual carriers in a composed multi carrier signal are leveled. \n
			:return: reference: RMS| PEAK
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:MCARrier:POWer:REFerence?')
		return Conversions.str_to_scalar_enum(response, enums.ArbMultCarrLevRef)

	def set_reference(self, reference: enums.ArbMultCarrLevRef) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:POWer:REFerence \n
		Snippet: driver.source.bb.arbitrary.mcarrier.power.set_reference(reference = enums.ArbMultCarrLevRef.PEAK) \n
		Defines the way the individual carriers in a composed multi carrier signal are leveled. \n
			:param reference: RMS| PEAK
		"""
		param = Conversions.enum_scalar_to_str(reference, enums.ArbMultCarrLevRef)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:MCARrier:POWer:REFerence {param}')



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal.RepeatedCapability import RepeatedCapability
from ..... import repcap


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class BitCls:
	"""Bit commands group definition. 5 total commands, 5 Subgroups, 0 group commands
	Repeated Capability: BitNumberNull, default value after init: BitNumberNull.Nr0"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("bit", core, parent)
		self._cmd_group.rep_cap = RepeatedCapability(self._cmd_group.group_name, 'repcap_bitNumberNull_get', 'repcap_bitNumberNull_set', repcap.BitNumberNull.Nr0)

	def repcap_bitNumberNull_set(self, bitNumberNull: repcap.BitNumberNull) -> None:
		"""Repeated Capability default value numeric suffix.
		This value is used, if you do not explicitely set it in the child set/get methods, or if you leave it to BitNumberNull.Default
		Default value after init: BitNumberNull.Nr0"""
		self._cmd_group.set_repcap_enum_value(bitNumberNull)

	def repcap_bitNumberNull_get(self) -> repcap.BitNumberNull:
		"""Returns the current default repeated capability for the child set/get methods"""
		# noinspection PyTypeChecker
		return self._cmd_group.get_repcap_enum_value()

	@property
	def condition(self):
		"""condition commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_condition'):
			from .Condition import ConditionCls
			self._condition = ConditionCls(self._core, self._cmd_group)
		return self._condition

	@property
	def enable(self):
		"""enable commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_enable'):
			from .Enable import EnableCls
			self._enable = EnableCls(self._core, self._cmd_group)
		return self._enable

	@property
	def ntransition(self):
		"""ntransition commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_ntransition'):
			from .Ntransition import NtransitionCls
			self._ntransition = NtransitionCls(self._core, self._cmd_group)
		return self._ntransition

	@property
	def ptransition(self):
		"""ptransition commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_ptransition'):
			from .Ptransition import PtransitionCls
			self._ptransition = PtransitionCls(self._core, self._cmd_group)
		return self._ptransition

	@property
	def event(self):
		"""event commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_event'):
			from .Event import EventCls
			self._event = EventCls(self._core, self._cmd_group)
		return self._event

	def clone(self) -> 'BitCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = BitCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from .......Internal.Core import Core
from .......Internal.CommandsGroup import CommandsGroup
from .......Internal import Conversions
from ....... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class FilterPyCls:
	"""FilterPy commands group definition. 9 total commands, 1 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("filterPy", core, parent)

	@property
	def parameter(self):
		"""parameter commands group. 0 Sub-classes, 8 commands."""
		if not hasattr(self, '_parameter'):
			from .Parameter import ParameterCls
			self._parameter = ParameterCls(self._core, self._cmd_group)
		return self._parameter

	# noinspection PyTypeChecker
	def get_type_py(self) -> enums.DmFilterA:
		"""SCPI: [SOURce<HW>]:BB:XMRadio:SATellite:FILTer:TYPE \n
		Snippet: value: enums.DmFilterA = driver.source.bb.xmRadio.satellite.filterPy.get_type_py() \n
		No command help available \n
			:return: type_py: No help available
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:XMRadio:SATellite:FILTer:TYPE?')
		return Conversions.str_to_scalar_enum(response, enums.DmFilterA)

	def set_type_py(self, type_py: enums.DmFilterA) -> None:
		"""SCPI: [SOURce<HW>]:BB:XMRadio:SATellite:FILTer:TYPE \n
		Snippet: driver.source.bb.xmRadio.satellite.filterPy.set_type_py(type_py = enums.DmFilterA.APCO25) \n
		No command help available \n
			:param type_py: No help available
		"""
		param = Conversions.enum_scalar_to_str(type_py, enums.DmFilterA)
		self._core.io.write(f'SOURce<HwInstance>:BB:XMRadio:SATellite:FILTer:TYPE {param}')

	def clone(self) -> 'FilterPyCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = FilterPyCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from typing import List

from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup
from ......Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class PowerCls:
	"""Power commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("power", core, parent)

	def get_points(self) -> int:
		"""SCPI: [SOURce<HW>]:CORRection:CSET:DATA:POWer:POINts \n
		Snippet: value: int = driver.source.correction.cset.data.power.get_points() \n
		Queries the number of level values in the selected table. The numerical suffix at SOURce must not be used for this
		command. \n
			:return: points: integer Range: 0 to 10000
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:CORRection:CSET:DATA:POWer:POINts?')
		return Conversions.str_to_int(response)

	def get_value(self) -> List[float]:
		"""SCPI: [SOURce<HW>]:CORRection:CSET:DATA:POWer \n
		Snippet: value: List[float] = driver.source.correction.cset.data.power.get_value() \n
		Transfers the level data to the table selected with [:SOURce<hw>]:CORRection:CSET[:SELect]. *RST does not affect data
		lists. The numerical suffix at SOURce must not be used for this command. \n
			:return: power: Power#1[, Power#2, ...]
		"""
		response = self._core.io.query_bin_or_ascii_float_list('SOURce<HwInstance>:CORRection:CSET:DATA:POWer?')
		return response

	def set_value(self, power: List[float]) -> None:
		"""SCPI: [SOURce<HW>]:CORRection:CSET:DATA:POWer \n
		Snippet: driver.source.correction.cset.data.power.set_value(power = [1.1, 2.2, 3.3]) \n
		Transfers the level data to the table selected with [:SOURce<hw>]:CORRection:CSET[:SELect]. *RST does not affect data
		lists. The numerical suffix at SOURce must not be used for this command. \n
			:param power: Power#1[, Power#2, ...]
		"""
		param = Conversions.list_to_csv_str(power)
		self._core.io.write(f'SOURce<HwInstance>:CORRection:CSET:DATA:POWer {param}')



from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup
from ......Internal import Conversions
from ......Internal.Utilities import trim_str_response


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class WaveformCls:
	"""Waveform commands group definition. 9 total commands, 2 Subgroups, 5 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("waveform", core, parent)

	@property
	def catalog(self):
		"""catalog commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_catalog'):
			from .Catalog import CatalogCls
			self._catalog = CatalogCls(self._core, self._cmd_group)
		return self._catalog

	@property
	def hddStreaming(self):
		"""hddStreaming commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_hddStreaming'):
			from .HddStreaming import HddStreamingCls
			self._hddStreaming = HddStreamingCls(self._core, self._cmd_group)
		return self._hddStreaming

	def delete(self, filename: str) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WAVeform:DELete \n
		Snippet: driver.source.bb.arbitrary.waveform.delete(filename = '1') \n
		Deletes the specified waveform file. If the file is not on the default path, the path must be specified at the same time.
		The file extension may be omitted. Only files with the file extension *.wv will be deleted. \n
			:param filename: string
		"""
		param = Conversions.value_to_quoted_str(filename)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:WAVeform:DELete {param}')

	def get_free(self) -> int:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WAVeform:FREE \n
		Snippet: value: int = driver.source.bb.arbitrary.waveform.get_free() \n
		Queries the free disk space on the default path of the instrument's harddisk. \n
			:return: free: integer Range: 0 to INT_MAX
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:WAVeform:FREE?')
		return Conversions.str_to_int(response)

	def get_points(self) -> int:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WAVeform:POINts \n
		Snippet: value: int = driver.source.bb.arbitrary.waveform.get_points() \n
		The command queries the number of samples in the waveform file selected using command ARB:WAV:SEL. Only the file name has
		to be entered. Only files with the file extension *.wv will be read out. \n
			:return: points: waveform filename Range: 0 to 1000
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:WAVeform:POINts?')
		return Conversions.str_to_int(response)

	def get_select(self) -> str:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WAVeform:SELect \n
		Snippet: value: str = driver.source.bb.arbitrary.waveform.get_select() \n
		Selects the waveform file. If the file is not on the default path, the path must be specified. The file extension may be
		omitted. Only files with the file extension *.wv will be created or loaded. \n
			:return: filename: string
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:WAVeform:SELect?')
		return trim_str_response(response)

	def set_select(self, filename: str) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WAVeform:SELect \n
		Snippet: driver.source.bb.arbitrary.waveform.set_select(filename = '1') \n
		Selects the waveform file. If the file is not on the default path, the path must be specified. The file extension may be
		omitted. Only files with the file extension *.wv will be created or loaded. \n
			:param filename: string
		"""
		param = Conversions.value_to_quoted_str(filename)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:WAVeform:SELect {param}')

	def get_tag(self) -> str:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WAVeform:TAG \n
		Snippet: value: str = driver.source.bb.arbitrary.waveform.get_tag() \n
		The command queries the content of the specified tag of the selected Waveform file. \n
			:return: tag: string
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:WAVeform:TAG?')
		return trim_str_response(response)

	def clone(self) -> 'WaveformCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = WaveformCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class AdjustCls:
	"""Adjust commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("adjust", core, parent)

	def get_value(self) -> int:
		"""SCPI: [SOURce]:ROSCillator:[INTernal]:ADJust:VALue \n
		Snippet: value: int = driver.source.roscillator.internal.adjust.get_value() \n
		Allows an application to shift the reference oscillator frequency by a small amount. The setting range depends on the
		reference oscillator type and its factory calibration value. Allowed are the following ranges:
			INTRO_CMD_HELP: Not affected are: \n
			- For TCXO oscillator: Max - Min = 1023
			- For OCXO oscillator: Max - Min = 65535 (option R&S SGT-B1 required.)  \n
			:return: value: integer Range: Min to Max
		"""
		response = self._core.io.query_str('SOURce:ROSCillator:INTernal:ADJust:VALue?')
		return Conversions.str_to_int(response)

	def set_value(self, value: int) -> None:
		"""SCPI: [SOURce]:ROSCillator:[INTernal]:ADJust:VALue \n
		Snippet: driver.source.roscillator.internal.adjust.set_value(value = 1) \n
		Allows an application to shift the reference oscillator frequency by a small amount. The setting range depends on the
		reference oscillator type and its factory calibration value. Allowed are the following ranges:
			INTRO_CMD_HELP: Not affected are: \n
			- For TCXO oscillator: Max - Min = 1023
			- For OCXO oscillator: Max - Min = 65535 (option R&S SGT-B1 required.)  \n
			:param value: integer Range: Min to Max
		"""
		param = Conversions.decimal_value_to_str(value)
		self._core.io.write(f'SOURce:ROSCillator:INTernal:ADJust:VALue {param}')

	def get_state(self) -> bool:
		"""SCPI: [SOURce]:ROSCillator:[INTernal]:ADJust:[STATe] \n
		Snippet: value: bool = driver.source.roscillator.internal.adjust.get_state() \n
		Determines whether the calibrated (OFF) or a user-defined (ON) adjustment value is used for fine adjustment of the
		frequency. If user-defined values are used, the instrument is no longer in the calibrated state. However, the calibration
		value is not changed and the instrument resumes the calibrated state after sending the command
		SOURce:ROSCillator:INTernal:ADJust:STATe OFF. \n
			:return: state: 0| 1| OFF| ON
		"""
		response = self._core.io.query_str('SOURce:ROSCillator:INTernal:ADJust:STATe?')
		return Conversions.str_to_bool(response)

	def set_state(self, state: bool) -> None:
		"""SCPI: [SOURce]:ROSCillator:[INTernal]:ADJust:[STATe] \n
		Snippet: driver.source.roscillator.internal.adjust.set_state(state = False) \n
		Determines whether the calibrated (OFF) or a user-defined (ON) adjustment value is used for fine adjustment of the
		frequency. If user-defined values are used, the instrument is no longer in the calibrated state. However, the calibration
		value is not changed and the instrument resumes the calibrated state after sending the command
		SOURce:ROSCillator:INTernal:ADJust:STATe OFF. \n
			:param state: 0| 1| OFF| ON
		"""
		param = Conversions.bool_to_str(state)
		self._core.io.write(f'SOURce:ROSCillator:INTernal:ADJust:STATe {param}')



"""Event-related methods and properties. Here you can set all the event handlers."""

from typing import Callable

from ..Internal import Core


class Events:
	"""Common Events class.
	Event-related methods and properties. Here you can set all the event handlers."""
	def __init__(self, core: Core):
		self._core = core

	@property
	def io_events_include_data(self) -> bool:
		"""Returns the current state of the io_events_include_data See the setter for more details."""
		return self._core.io.io_events_include_data

	@io_events_include_data.setter
	def io_events_include_data(self, value: bool) -> None:
		"""If True, the on_write and on_read events include also the sent/received data.
		Default value is False, to avoid handling potentially big data."""
		self._core.io.io_events_include_data = value

	@property
	def before_write_handler(self) -> Callable:
		"""Returns the handler of before_write events. \n
		:return: current ``before_write_handler``"""
		return self._core.io.before_write_handler

	@before_write_handler.setter
	def before_write_handler(self, handler: Callable) -> None:
		"""Sets handler for before_write events.
		The before_write event is invoked before each write operation (only once, not for every chunk)
		Event prototype: handler(io: Instrument, cmd: str)
		:param handler: new handler"""
		self._core.io.before_write_handler = handler

	@property
	def on_write_handler(self) -> Callable:
		"""Returns the handler of on_write events. \n
		:return: current ``on_write_handler``"""
		return self._core.io.on_write_handler

	@on_write_handler.setter
	def on_write_handler(self, handler: Callable) -> None:
		"""Sets handler for on_write events.
		The on_write event is invoked every time the driver performs a write operation to the instrument (for each write chunk)
		Event arguments type: IoTransferEventArgs
		By default, the event_args do not contain the actual data sent. If you wish to receive them, set the driver.Events.io_events_include_data to True \n
		:param handler: new handler for all write operations"""
		self._core.io.on_write_handler = handler

	@property
	def on_read_handler(self) -> Callable:
		"""Returns the handler of on_read events. \n
		:return: current ``on_read_handler``"""
		return self._core.io.on_read_handler

	@on_read_handler.setter
	def on_read_handler(self, handler: Callable) -> None:
		"""Sets handler for on_read events.
		The on_read event is invoked every time the driver performs a read operation to the instrument.
		Event arguments type: IoTransferEventArgs
		By default, the event_args do not contain the actual data sent. If you wish to receive them, set the driver.Events.io_events_include_data to True \n
		:param handler: new handler for all read operations"""
		self._core.io.on_read_handler = handler

	@property
	def before_query_handler(self) -> Callable:
		"""Returns the handler of before_query events. \n
		:return: current ``before_query_handler``"""
		return self._core.io.before_query_handler

	@before_query_handler.setter
	def before_query_handler(self, handler: Callable) -> None:
		"""Sets handler for before_query events.
		The before_query event is invoked before each query operation (only once, not for every chunk)
		Event prototype: handler(io: Instrument, query: str)
		:param handler: new handler"""
		self._core.io.before_query_handler = handler

	def sync_from(self, source: 'Events') -> None:
		"""Synchronises these Events with the source."""
		self.before_query_handler = source.before_query_handler
		self.before_write_handler = source.before_write_handler
		self.io_events_include_data = source.io_events_include_data
		self.on_read_handler = source.on_read_handler
		self.on_write_handler = source.on_write_handler



from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup
from ......Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class ErrorCls:
	"""Error commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("error", core, parent)

	def get_max(self) -> float:
		"""SCPI: [SOURce<HW>]:IQ:DPD:OUTPut:ERRor:MAX \n
		Snippet: value: float = driver.source.iq.dpd.output.error.get_max() \n
		Sets the allowed maximum error. \n
			:return: maximum_error: float Range: 0.01 to 1
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:DPD:OUTPut:ERRor:MAX?')
		return Conversions.str_to_float(response)

	def set_max(self, maximum_error: float) -> None:
		"""SCPI: [SOURce<HW>]:IQ:DPD:OUTPut:ERRor:MAX \n
		Snippet: driver.source.iq.dpd.output.error.set_max(maximum_error = 1.0) \n
		Sets the allowed maximum error. \n
			:param maximum_error: float Range: 0.01 to 1
		"""
		param = Conversions.decimal_value_to_str(maximum_error)
		self._core.io.write(f'SOURce<HwInstance>:IQ:DPD:OUTPut:ERRor:MAX {param}')

	def get_value(self) -> float:
		"""SCPI: [SOURce<HW>]:IQ:DPD:OUTPut:ERRor \n
		Snippet: value: float = driver.source.iq.dpd.output.error.get_value() \n
		Queries the resulting level error. \n
			:return: achieved_error: float
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:DPD:OUTPut:ERRor?')
		return Conversions.str_to_float(response)



from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class TsignalCls:
	"""Tsignal commands group definition. 16 total commands, 4 Subgroups, 0 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("tsignal", core, parent)

	@property
	def awgn(self):
		"""awgn commands group. 1 Sub-classes, 0 commands."""
		if not hasattr(self, '_awgn'):
			from .Awgn import AwgnCls
			self._awgn = AwgnCls(self._core, self._cmd_group)
		return self._awgn

	@property
	def ciq(self):
		"""ciq commands group. 1 Sub-classes, 2 commands."""
		if not hasattr(self, '_ciq'):
			from .Ciq import CiqCls
			self._ciq = CiqCls(self._core, self._cmd_group)
		return self._ciq

	@property
	def rectangle(self):
		"""rectangle commands group. 1 Sub-classes, 4 commands."""
		if not hasattr(self, '_rectangle'):
			from .Rectangle import RectangleCls
			self._rectangle = RectangleCls(self._core, self._cmd_group)
		return self._rectangle

	@property
	def sine(self):
		"""sine commands group. 1 Sub-classes, 3 commands."""
		if not hasattr(self, '_sine'):
			from .Sine import SineCls
			self._sine = SineCls(self._core, self._cmd_group)
		return self._sine

	def clone(self) -> 'TsignalCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = TsignalCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from typing import List

from .......Internal.Core import Core
from .......Internal.CommandsGroup import CommandsGroup
from .......Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class DataCls:
	"""Data commands group definition. 4 total commands, 0 Subgroups, 4 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("data", core, parent)

	def get_catalog(self) -> List[str]:
		"""SCPI: [SOURce<HW>]:IQ:DPD:SHAPing:NORMalized:DATA:CATalog \n
		Snippet: value: List[str] = driver.source.iq.dpd.shaping.normalized.data.get_catalog() \n
		Queries the available files with normalized data in the default directory. Only files with the file extension *.dpd_norm
		are listed. \n
			:return: catalog: string
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:DPD:SHAPing:NORMalized:DATA:CATalog?')
		return Conversions.str_to_str_list(response)

	def load(self, filename: str) -> None:
		"""SCPI: [SOURce<HW>]:IQ:DPD:SHAPing:NORMalized:DATA:LOAD \n
		Snippet: driver.source.iq.dpd.shaping.normalized.data.load(filename = '1') \n
		Loads the selected file. \n
			:param filename: string
		"""
		param = Conversions.value_to_quoted_str(filename)
		self._core.io.write(f'SOURce<HwInstance>:IQ:DPD:SHAPing:NORMalized:DATA:LOAD {param}')

	def set_store(self, filename: str) -> None:
		"""SCPI: [SOURce<HW>]:IQ:DPD:SHAPing:NORMalized:DATA:STORe \n
		Snippet: driver.source.iq.dpd.shaping.normalized.data.set_store(filename = '1') \n
		Saves the normalized data in a file. \n
			:param filename: string
		"""
		param = Conversions.value_to_quoted_str(filename)
		self._core.io.write(f'SOURce<HwInstance>:IQ:DPD:SHAPing:NORMalized:DATA:STORe {param}')

	def get_value(self) -> bytes:
		"""SCPI: [SOURce<HW>]:IQ:DPD:SHAPing:NORMalized:DATA \n
		Snippet: value: bytes = driver.source.iq.dpd.shaping.normalized.data.get_value() \n
		Defines the normalized predistortion function in a raw data format (binary data) . \n
			:return: data: #LengthNoBytesNoBytesNormData # The binary data must start with the sign # LengthNoBytes ASCII format Sets the length of NoBytes, i.e. the number of digits used to write NoBytes NoBytes An ASCII integer value that specifies the number of bytes that follow in the NormData part Each of the NormData parameters is coded with 8 bytes. Then the number of bytes NoBytes is calculated as: NoBytes = 8 + 8 + n(8+8+8) , where n is the number of points NoPoints. NormData PinMaxNoPoints{VinVmaxDeltaVDeltaPhase} Values in binary format, describing the maximum absolute input power Pinmax, the number of subsequent points n and the normalized values Vin/Vmax, ΔV/V, ΔPhase [deg].
		"""
		response = self._core.io.query_bin_block('SOURce<HwInstance>:IQ:DPD:SHAPing:NORMalized:DATA?')
		return response

	def set_value(self, data: bytes) -> None:
		"""SCPI: [SOURce<HW>]:IQ:DPD:SHAPing:NORMalized:DATA \n
		Snippet: driver.source.iq.dpd.shaping.normalized.data.set_value(data = b'ABCDEFGH') \n
		Defines the normalized predistortion function in a raw data format (binary data) . \n
			:param data: #LengthNoBytesNoBytesNormData # The binary data must start with the sign # LengthNoBytes ASCII format Sets the length of NoBytes, i.e. the number of digits used to write NoBytes NoBytes An ASCII integer value that specifies the number of bytes that follow in the NormData part Each of the NormData parameters is coded with 8 bytes. Then the number of bytes NoBytes is calculated as: NoBytes = 8 + 8 + n(8+8+8) , where n is the number of points NoPoints. NormData PinMaxNoPoints{VinVmaxDeltaVDeltaPhase} Values in binary format, describing the maximum absolute input power Pinmax, the number of subsequent points n and the normalized values Vin/Vmax, ΔV/V, ΔPhase [deg].
		"""
		self._core.io.write_bin_block('SOURce<HwInstance>:IQ:DPD:SHAPing:NORMalized:DATA ', data)



from typing import ClassVar, List

from .Internal.Core import Core
from .Internal.InstrumentErrors import RsInstrException
from .Internal.CommandsGroup import CommandsGroup
from .Internal.VisaSession import VisaSession
from datetime import datetime, timedelta
from .Internal import Conversions
from .Internal.StructBase import StructBase
from .Internal.ArgStruct import ArgStruct
from . import repcap
from .Internal.RepeatedCapability import RepeatedCapability


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class RsSgt:
	"""794 total commands, 16 Subgroups, 4 group commands"""
	_driver_options = "SupportedInstrModels = SMW/SMBV/SGT/SMA/SMM/SMCV, SupportedIdnPatterns = SMW/SMBV/SGT/SMA/SMM/SMCV, SimulationIdnString = 'Rohde&Schwarz,SGT100A,100001,4.90.109.0029'"
	_global_logging_relative_timestamp: ClassVar[datetime] = None
	_global_logging_target_stream: ClassVar = None

	def __init__(self, resource_name: str, id_query: bool = True, reset: bool = False, options: str = None, direct_session: object = None):
		"""Initializes new RsSgt session. \n
		Parameter options tokens examples:
			- ``Simulate=True`` - starts the session in simulation mode. Default: ``False``
			- ``SelectVisa=socket`` - uses no VISA implementation for socket connections - you do not need any VISA-C installation
			- ``SelectVisa=rs`` - forces usage of RohdeSchwarz Visa
			- ``SelectVisa=ivi`` - forces usage of National Instruments Visa
			- ``QueryInstrumentStatus = False`` - same as ``driver.utilities.instrument_status_checking = False``. Default: ``True``
			- ``WriteDelay = 20, ReadDelay = 5`` - Introduces delay of 20ms before each write and 5ms before each read. Default: ``0ms`` for both
			- ``OpcWaitMode = OpcQuery`` - mode for all the opc-synchronised write/reads. Other modes: StbPolling, StbPollingSlow, StbPollingSuperSlow. Default: ``StbPolling``
			- ``AddTermCharToWriteBinBLock = True`` - Adds one additional LF to the end of the binary data (some instruments require that). Default: ``False``
			- ``AssureWriteWithTermChar = True`` - Makes sure each command/query is terminated with termination character. Default: Interface dependent
			- ``TerminationCharacter = "\\r"`` - Sets the termination character for reading. Default: ``\\n`` (LineFeed or LF)
			- ``DataChunkSize = 10E3`` - Maximum size of one write/read segment. If transferred data is bigger, it is split to more segments. Default: ``1E6`` bytes
			- ``OpcTimeout = 10000`` - same as driver.utilities.opc_timeout = 10000. Default: ``30000ms``
			- ``VisaTimeout = 5000`` - same as driver.utilities.visa_timeout = 5000. Default: ``10000ms``
			- ``ViClearExeMode = Disabled`` - viClear() execution mode. Default: ``execute_on_all``
			- ``OpcQueryAfterWrite = True`` - same as driver.utilities.opc_query_after_write = True. Default: ``False``
			- ``StbInErrorCheck = False`` - if true, the driver checks errors with *STB? If false, it uses SYST:ERR?. Default: ``True``
			- ``LoggingMode = On`` - Sets the logging status right from the start. Default: ``Off``
			- ``LoggingName = 'MyDevice'`` - Sets the name to represent the session in the log entries. Default: ``'resource_name'``
			- ``LogToGlobalTarget = True`` - Sets the logging target to the class-property previously set with RsSgt.set_global_logging_target() Default: ``False``
			- ``LoggingToConsole = True`` - Immediately starts logging to the console. Default: False
			- ``LoggingToUdp = True`` - Immediately starts logging to the UDP port. Default: False
			- ``LoggingUdpPort = 49200`` - UDP port to log to. Default: 49200
		:param resource_name: VISA resource name, e.g. 'TCPIP::192.168.2.1::INSTR'
		:param id_query: if True, the instrument's model name is verified against the models supported by the driver and eventually throws an exception.
		:param reset: Resets the instrument (sends *RST command) and clears its status sybsystem.
		:param options: string tokens alternating the driver settings.
		:param direct_session: Another driver object or pyVisa object to reuse the session instead of opening a new session."""
		self._core = Core(resource_name, id_query, reset, RsSgt._driver_options, options, direct_session)
		self._core.driver_version = '4.90.109.0029'
		self._options = options
		self._add_all_global_repcaps()
		self._custom_properties_init()
		self.utilities.default_instrument_setup()
		# noinspection PyTypeChecker
		self._cmd_group = CommandsGroup("ROOT", self._core, None)

	@classmethod
	def from_existing_session(cls, session: object, options: str = None) -> 'RsSgt':
		"""Creates a new RsSgt object with the entered 'session' reused. \n
		:param session: can be another driver or a direct pyvisa session.
		:param options: string tokens alternating the driver settings."""
		# noinspection PyTypeChecker
		resource_name = None
		if hasattr(session, 'resource_name'):
			resource_name = getattr(session, 'resource_name')
		return cls(resource_name, False, False, options, session)
		
	@classmethod
	def set_global_logging_target(cls, target) -> None:
		"""Sets global common target stream that each instance can use. To use it, call the following: io.utilities.logger.set_logging_target_global().
		If an instance uses global logging target, it automatically uses the global relative timestamp (if set).
		You can set the target to None to invalidate it."""
		cls._global_logging_target_stream = target

	@classmethod
	def get_global_logging_target(cls):
		"""Returns global common target stream."""
		return cls._global_logging_target_stream

	@classmethod
	def set_global_logging_relative_timestamp(cls, timestamp: datetime) -> None:
		"""Sets global common relative timestamp for log entries. To use it, call the following: io.utilities.logger.set_relative_timestamp_global()"""
		cls._global_logging_relative_timestamp = timestamp

	@classmethod
	def set_global_logging_relative_timestamp_now(cls) -> None:
		"""Sets global common relative timestamp for log entries to this moment.
		To use it, call the following: io.utilities.logger.set_relative_timestamp_global()."""
		cls._global_logging_relative_timestamp = datetime.now()

	@classmethod
	def clear_global_logging_relative_timestamp(cls) -> None:
		"""Clears the global relative timestamp. After this, all the instances using the global relative timestamp continue logging with the absolute timestamps."""
		# noinspection PyTypeChecker
		cls._global_logging_relative_timestamp = None

	@classmethod
	def get_global_logging_relative_timestamp(cls) -> datetime or None:
		"""Returns global common relative timestamp for log entries."""
		return cls._global_logging_relative_timestamp

	def __str__(self) -> str:
		if self._core.io:
			return f"RsSgt session '{self._core.io.resource_name}'"
		else:
			return f"RsSgt with session closed"

	def get_total_execution_time(self) -> timedelta:
		"""Returns total time spent by the library on communicating with the instrument.
		This time is always shorter than get_total_time(), since it does not include gaps between the communication.
		You can reset this counter with reset_time_statistics()."""
		return self._core.io.total_execution_time

	def get_total_time(self) -> timedelta:
		"""Returns total time spent by the library on communicating with the instrument.
		This time is always shorter than get_total_time(), since it does not include gaps between the communication.
		You can reset this counter with reset_time_statistics()."""
		return datetime.now() - self._core.io.total_time_startpoint

	def reset_time_statistics(self) -> None:
		"""Resets all execution and total time counters. Affects the results of get_total_time() and get_total_execution_time()"""
		self._core.io.reset_time_statistics()

	@staticmethod
	def assert_minimum_version(min_version: str) -> None:
		"""Asserts that the driver version fulfills the minimum required version you have entered.
		This way you make sure your installed driver is of the entered version or newer."""
		min_version_list = min_version.split('.')
		curr_version_list = '4.90.109.0029'.split('.')
		count_min = len(min_version_list)
		count_curr = len(curr_version_list)
		count = count_min if count_min < count_curr else count_curr
		for i in range(count):
			minimum = int(min_version_list[i])
			curr = int(curr_version_list[i])
			if curr > minimum:
				break
			if curr < minimum:
				raise RsInstrException(f"Assertion for minimum RsSgt version failed. Current version: '4.90.109.0029', minimum required version: '{min_version}'")

	@staticmethod
	def list_resources(expression: str = '?*::INSTR', visa_select: str = None) -> List[str]:
		"""Finds all the resources defined by the expression
			- '?*' - matches all the available instruments
			- 'USB::?*' - matches all the USB instruments
			- 'TCPIP::192?*' - matches all the LAN instruments with the IP address starting with 192
		:param expression: see the examples in the function
		:param visa_select: optional parameter selecting a specific VISA. Examples: '@ivi', '@rs'
		"""
		rm = VisaSession.get_resource_manager(visa_select)
		resources = rm.list_resources(expression)
		rm.close()
		# noinspection PyTypeChecker
		return resources

	def close(self) -> None:
		"""Closes the active RsSgt session."""
		self._core.io.close()

	def get_session_handle(self) -> object:
		"""Returns the underlying session handle."""
		return self._core.get_session_handle()

	def _add_all_global_repcaps(self) -> None:
		"""Adds all the repcaps defined as global to the instrument's global repcaps dictionary."""
		self._core.io.add_global_repcap('<HwInstance>', RepeatedCapability("ROOT", 'repcap_hwInstance_get', 'repcap_hwInstance_set', repcap.HwInstance.InstA))

	def repcap_hwInstance_get(self) -> repcap.HwInstance:
		"""Returns Global Repeated capability HwInstance"""
		return self._core.io.get_global_repcap_value('<HwInstance>')

	def repcap_hwInstance_set(self, value: repcap.HwInstance) -> None:
		"""Sets Global Repeated capability HwInstance
		Default value after init: HwInstance.InstA"""
		self._core.io.set_global_repcap_value('<HwInstance>', value)

	def _custom_properties_init(self) -> None:
		"""Adds all the interfaces that are custom for the driver."""
		from .CustomFiles.utilities import Utilities
		self.utilities = Utilities(self._core)
		from .CustomFiles.events import Events
		self.events = Events(self._core)
		from .CustomFiles.arb_files import ArbFiles
		self.arb_files = ArbFiles(self._core)
		from .CustomFiles.digital_modulation import DigitalModulation
		self.digital_modulation = DigitalModulation(self._core)
		
	def _sync_to_custom_properties(self, cloned: 'RsSgt') -> None:
		"""Synchronises the state of all the custom properties to the entered object."""
		cloned.utilities.sync_from(self.utilities)
		cloned.events.sync_from(self.events)
		cloned.arb_files.sync_from(self.arb_files)
		cloned.digital_modulation.sync_from(self.digital_modulation)

	@property
	def calibration(self):
		"""calibration commands group. 6 Sub-classes, 0 commands."""
		if not hasattr(self, '_calibration'):
			from .Implementations.Calibration import CalibrationCls
			self._calibration = CalibrationCls(self._core, self._cmd_group)
		return self._calibration

	@property
	def clock(self):
		"""clock commands group. 3 Sub-classes, 0 commands."""
		if not hasattr(self, '_clock'):
			from .Implementations.Clock import ClockCls
			self._clock = ClockCls(self._core, self._cmd_group)
		return self._clock

	@property
	def connector(self):
		"""connector commands group. 2 Sub-classes, 0 commands."""
		if not hasattr(self, '_connector'):
			from .Implementations.Connector import ConnectorCls
			self._connector = ConnectorCls(self._core, self._cmd_group)
		return self._connector

	@property
	def device(self):
		"""device commands group. 1 Sub-classes, 0 commands."""
		if not hasattr(self, '_device'):
			from .Implementations.Device import DeviceCls
			self._device = DeviceCls(self._core, self._cmd_group)
		return self._device

	@property
	def diagnostic(self):
		"""diagnostic commands group. 6 Sub-classes, 0 commands."""
		if not hasattr(self, '_diagnostic'):
			from .Implementations.Diagnostic import DiagnosticCls
			self._diagnostic = DiagnosticCls(self._core, self._cmd_group)
		return self._diagnostic

	@property
	def display(self):
		"""display commands group. 2 Sub-classes, 0 commands."""
		if not hasattr(self, '_display'):
			from .Implementations.Display import DisplayCls
			self._display = DisplayCls(self._core, self._cmd_group)
		return self._display

	@property
	def formatPy(self):
		"""formatPy commands group. 0 Sub-classes, 3 commands."""
		if not hasattr(self, '_formatPy'):
			from .Implementations.FormatPy import FormatPyCls
			self._formatPy = FormatPyCls(self._core, self._cmd_group)
		return self._formatPy

	@property
	def memory(self):
		"""memory commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_memory'):
			from .Implementations.Memory import MemoryCls
			self._memory = MemoryCls(self._core, self._cmd_group)
		return self._memory

	@property
	def massMemory(self):
		"""massMemory commands group. 4 Sub-classes, 8 commands."""
		if not hasattr(self, '_massMemory'):
			from .Implementations.MassMemory import MassMemoryCls
			self._massMemory = MassMemoryCls(self._core, self._cmd_group)
		return self._massMemory

	@property
	def output(self):
		"""output commands group. 3 Sub-classes, 1 commands."""
		if not hasattr(self, '_output'):
			from .Implementations.Output import OutputCls
			self._output = OutputCls(self._core, self._cmd_group)
		return self._output

	@property
	def sconfiguration(self):
		"""sconfiguration commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_sconfiguration'):
			from .Implementations.Sconfiguration import SconfigurationCls
			self._sconfiguration = SconfigurationCls(self._core, self._cmd_group)
		return self._sconfiguration

	@property
	def source(self):
		"""source commands group. 14 Sub-classes, 2 commands."""
		if not hasattr(self, '_source'):
			from .Implementations.Source import SourceCls
			self._source = SourceCls(self._core, self._cmd_group)
		return self._source

	@property
	def status(self):
		"""status commands group. 3 Sub-classes, 1 commands."""
		if not hasattr(self, '_status'):
			from .Implementations.Status import StatusCls
			self._status = StatusCls(self._core, self._cmd_group)
		return self._status

	@property
	def system(self):
		"""system commands group. 29 Sub-classes, 28 commands."""
		if not hasattr(self, '_system'):
			from .Implementations.System import SystemCls
			self._system = SystemCls(self._core, self._cmd_group)
		return self._system

	@property
	def test(self):
		"""test commands group. 9 Sub-classes, 3 commands."""
		if not hasattr(self, '_test'):
			from .Implementations.Test import TestCls
			self._test = TestCls(self._core, self._cmd_group)
		return self._test

	@property
	def unit(self):
		"""unit commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_unit'):
			from .Implementations.Unit import UnitCls
			self._unit = UnitCls(self._core, self._cmd_group)
		return self._unit

	def get_ffast(self) -> float:
		"""SCPI: FFASt \n
		Snippet: value: float = driver.get_ffast() \n
		Special command to set the RF output frequency with minimum latency. No unit (e.g. Hz) allowed. Bypasses the status
		system so command *OPC? cannot be appended. \n
			:return: freq: float
		"""
		response = self._core.io.query_str('FFASt?')
		return Conversions.str_to_float(response)

	def set_ffast(self, freq: float) -> None:
		"""SCPI: FFASt \n
		Snippet: driver.set_ffast(freq = 1.0) \n
		Special command to set the RF output frequency with minimum latency. No unit (e.g. Hz) allowed. Bypasses the status
		system so command *OPC? cannot be appended. \n
			:param freq: float
		"""
		param = Conversions.decimal_value_to_str(freq)
		self._core.io.write(f'FFASt {param}')

	# noinspection PyTypeChecker
	class LockStruct(StructBase):  # From ReadStructDefinition CmdPropertyTemplate.xml
		"""Structure for reading output parameters. Fields: \n
			- Lock_Request_Id: float: Number 0 test query to prove whether the instrument is locked Controller ID request lock from the controller with the specified Controller ID
			- Value: float: Number 0 request refused; the instrument is already locked to other Lock Request Id, i.e. to another controller 1 request granted"""
		__meta_args_list = [
			ArgStruct.scalar_float('Lock_Request_Id'),
			ArgStruct.scalar_float('Value')]

		def __init__(self):
			StructBase.__init__(self, self)
			self.Lock_Request_Id: float = None
			self.Value: float = None

	def get_lock(self) -> LockStruct:
		"""SCPI: LOCK \n
		Snippet: value: LockStruct = driver.get_lock() \n
		Sends a lock request ID which uniquely identifies the controller to the instrument. \n
			:return: structure: for return value, see the help for LockStruct structure arguments.
		"""
		return self._core.io.query_struct('LOCK?', self.__class__.LockStruct())

	def get_pfast(self) -> float:
		"""SCPI: PFASt \n
		Snippet: value: float = driver.get_pfast() \n
		Special command to set the RF output level with minimum latency at the RF output connector. This value does not consider
		a specified offset. No unit (e.g. dBm) allowed. Bypasses the status system so command *OPC? cannot be appended. \n
			:return: power: float
		"""
		response = self._core.io.query_str('PFASt?')
		return Conversions.str_to_float(response)

	def set_pfast(self, power: float) -> None:
		"""SCPI: PFASt \n
		Snippet: driver.set_pfast(power = 1.0) \n
		Special command to set the RF output level with minimum latency at the RF output connector. This value does not consider
		a specified offset. No unit (e.g. dBm) allowed. Bypasses the status system so command *OPC? cannot be appended. \n
			:param power: float
		"""
		param = Conversions.decimal_value_to_str(power)
		self._core.io.write(f'PFASt {param}')

	def unlock(self, unlock_id: float) -> None:
		"""SCPI: UNLock \n
		Snippet: driver.unlock(unlock_id = 1.0) \n
		Unlocks an instrument locked to a controller with Controller ID = <Unlock Id>. \n
			:param unlock_id: Number Unlock ID which uniquely identifies the controller to the instrument. The value must match the Controller ID Lock Request Id set with the command [CMDLINKRESOLVED #Lock CMDLINKRESOLVED]. 0 Clear lock regardless of locking state
		"""
		param = Conversions.decimal_value_to_str(unlock_id)
		self._core.io.write(f'UNLock {param}')

	def clone(self) -> 'RsSgt':
		"""Creates a deep copy of the RsSgt object. Also copies:
			- All the existing Global repeated capability values
			- All the default group repeated capabilities setting \n
		Does not check the *IDN? response, and does not perform Reset.
		After cloning, you can set all the repeated capabilities settings independentely from the original group.
		Calling close() on the new object does not close the original VISA session"""
		cloned = RsSgt.from_existing_session(self.get_session_handle(), self._options)
		self._cmd_group.synchronize_repcaps(cloned)
		cloned.repcap_hwInstance_set(self.repcap_hwInstance_get())
		self._sync_to_custom_properties(cloned)
		return cloned

	def restore_all_repcaps_to_default(self) -> None:
		"""Sets all the Group and Global repcaps to their initial values"""
		self._cmd_group.restore_repcaps()
		self.repcap_hwInstance_set(repcap.HwInstance.InstA)



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions
from ..... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class PowerCls:
	"""Power commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("power", core, parent)

	# noinspection PyTypeChecker
	def get_via(self) -> enums.IqOutDispViaType:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:POWer:VIA \n
		Snippet: value: enums.IqOutDispViaType = driver.source.iq.output.power.get_via() \n
		No command help available \n
			:return: via: No help available
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:OUTPut:POWer:VIA?')
		return Conversions.str_to_scalar_enum(response, enums.IqOutDispViaType)

	def set_via(self, via: enums.IqOutDispViaType) -> None:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:POWer:VIA \n
		Snippet: driver.source.iq.output.power.set_via(via = enums.IqOutDispViaType.LEVel) \n
		No command help available \n
			:param via: No help available
		"""
		param = Conversions.enum_scalar_to_str(via, enums.IqOutDispViaType)
		self._core.io.write(f'SOURce<HwInstance>:IQ:OUTPut:POWer:VIA {param}')



"""See the docstring for the StreamWriter class."""

from enum import Flag
from typing import AnyStr
from io import BytesIO, StringIO

from .Utilities import size_to_kb_mb_string
from .InstrumentErrors import RsInstrException


class Type(Flag):
	"""Defines type of the stream - variable or file."""
	Variable = 1
	Forget = 2
	File = 4
	FileAppend = 12


class StreamWriter:
	"""Lightweight stream writer implementation. Data target can be: \n
	- bytes
	- string
	- file"""

	def __init__(self, binary: bool, target: Type, meta_data=None):
		"""Initializes StreamWriter instance.\n
		:param binary: True: Binary data, False: ASCII data
		:param target: Target for the stream. Variable / File (FileAppend)
		:param meta_data: Only valid for File and FileAppend - define file path as string:
		For Type.File, data must be string with file path. If the file exists, it will be overwritten.
		For Type.FileAppend, data must be string with file path. If the file exists, it will be appended."""
		self._binary: bool = binary
		self._written_len: int = 0
		self._target = target

		if Type.Variable in self._target:
			assert meta_data is None, f'You can not define input meta_data for a Variable StreamWriter.'
			self._data = BytesIO() if binary else StringIO()
		elif Type.Forget in self._target:
			self._data: AnyStr = ''
		elif Type.File in self._target:
			assert isinstance(meta_data, str), f'Additional data must be of string type (file path). Actual type: {type(meta_data)}'
			self._file_path = meta_data
			mode = 'w' if self._target == Type.File else 'a'
			mode += 'b' if self._binary else ''
			self._data = open(self._file_path, mode)
		else:
			raise RsInstrException(f'StreamWriter unknown target {target}')

	@classmethod
	def as_bin_var(cls) -> 'StreamWriter':
		"""Creates new StreamWriter with bytes variable."""
		return cls(True, Type.Variable)

	@classmethod
	def as_string_var(cls) -> 'StreamWriter':
		"""Creates new StreamWriter with string variable."""
		return cls(False, Type.Variable)

	@classmethod
	def as_forget(cls) -> 'StreamWriter':
		"""Creates new StreamWriter which writes to nowhere - forgets the data."""
		return cls(False, Type.Forget)

	@classmethod
	def as_bin_file(cls, file_path: str, append: bool = False) -> 'StreamWriter':
		"""Creates new StreamWriter to binary file.
		:param file_path: [str] Path to the file.
		:param append: Optional [bool] If True, the content is appended to the existing content."""
		return cls(True, Type.FileAppend if append else Type.File, file_path)

	@classmethod
	def as_text_file(cls, file_path: str, append: bool = False) -> 'StreamWriter':
		"""Creates new StreamWriter to text file.
		:param file_path: [str] Path to the file.
		:param append: Optional [bool] If True, the content is appended to the existing content."""
		return cls(False, Type.FileAppend if append else Type.File, file_path)

	def __str__(self):
		if Type.Variable in self._target:
			mode = 'binary' if self._binary else 'string'
			return f'StreamWriter {mode} variable, current size {size_to_kb_mb_string(len(self), True)}'
		if Type.File in self._target:
			mode = 'binary' if self._binary else 'text'
			append = ' appended' if Type.FileAppend in self._target else ''
			return f'StreamWriter {mode} file{append}, current{append} size {size_to_kb_mb_string(len(self), True)}, file: {self._file_path}'
		if Type.Forget in self._target:
			return 'StreamWriter to nowhere.'

	def __len__(self):
		"""Returns remaining length."""
		return self._written_len

	def __enter__(self):
		return self

	def __exit__(self, exception_type, exception_value, traceback):
		self.close()

	@property
	def binary(self) -> bool:
		"""Returns true, if the data held is binary.
		File streams are always binary."""
		return self._binary

	def write(self, data: AnyStr) -> None:
		"""Writes chunk to the stream.
			- For Type.Bytes data must be bytes.
			- For Type.String, data must be string.
			- For Type.File and Type.FileAppend, data must be bytes."""
		if Type.Forget in self._target:
			self._written_len += len(data)
			return

		assert self._data is not None, 'StreamWriter buffer is invalid. You have probably closed it already.'
		if self._binary:
			assert isinstance(data, bytes), f'Bytes data is required. Actual type: {type(data)}. {self}'
		else:
			assert isinstance(data, str), f'String data is required. Actual type: {type(data)}. {self}'
		if Type.Variable in self._target:
			self._data.write(data)
		elif Type.File in self._target:
			self._data.write(data)
		self._written_len += len(data)

	def switch_to_string_data(self, encoding: str) -> None:
		"""Switches from binary to string data.
		For variables, the current content is converted to string using the provided encoding.
		For files, they are closed and reopened as for appended text writing."""
		if self._binary is False:
			return
		self._binary = False
		if Type.Variable in self._target:
			self._data = StringIO(self.content.decode(encoding))
		elif Type.File in self._target:
			self._data.close()
			self._data = open(self._file_path, 'a')

	# noinspection PyTypeChecker
	@property
	def content(self) -> AnyStr:
		"""Returns content of the writer. Only works with variable types."""
		if self._target == Type.Forget:
			return ''
		if self._target != Type.Variable:
			raise RsInstrException(f'Can not return content for the current {self}')
		# noinspection PyTypeChecker
		if not self._data:
			return None
		self._data.seek(0)
		ret_val = self._data.read()
		self._data.close()
		return ret_val

	@property
	def written_len(self) -> int:
		"""Returns number of bytes written to the stream since its creation."""
		return self._written_len

	def close(self) -> None:
		"""Closes the StreamWriter. You can not use its instance afterwards."""
		if Type.File in self._target and self._data:
			self._data.close()
		self._data = None



from typing import List

from .......Internal.Core import Core
from .......Internal.CommandsGroup import CommandsGroup
from .......Internal import Conversions
from .......Internal.Utilities import trim_str_response


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class ConfigureCls:
	"""Configure commands group definition. 14 total commands, 5 Subgroups, 5 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("configure", core, parent)

	@property
	def blank(self):
		"""blank commands group. 1 Sub-classes, 0 commands."""
		if not hasattr(self, '_blank'):
			from .Blank import BlankCls
			self._blank = BlankCls(self._core, self._cmd_group)
		return self._blank

	@property
	def clock(self):
		"""clock commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_clock'):
			from .Clock import ClockCls
			self._clock = ClockCls(self._core, self._cmd_group)
		return self._clock

	@property
	def level(self):
		"""level commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_level'):
			from .Level import LevelCls
			self._level = LevelCls(self._core, self._cmd_group)
		return self._level

	@property
	def marker(self):
		"""marker commands group. 0 Sub-classes, 3 commands."""
		if not hasattr(self, '_marker'):
			from .Marker import MarkerCls
			self._marker = MarkerCls(self._core, self._cmd_group)
		return self._marker

	@property
	def segment(self):
		"""segment commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_segment'):
			from .Segment import SegmentCls
			self._segment = SegmentCls(self._core, self._cmd_group)
		return self._segment

	def get_catalog(self) -> List[str]:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WSEGment:CONFigure:CATalog \n
		Snippet: value: List[str] = driver.source.bb.arbitrary.wsegment.configure.get_catalog() \n
		Queries the available configuration files in the specified default directory. The configuration files are used to create
		multi segment waveform files. \n
			:return: catalog: string
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:WSEGment:CONFigure:CATalog?')
		return Conversions.str_to_str_list(response)

	def get_comment(self) -> str:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WSEGment:CONFigure:COMMent \n
		Snippet: value: str = driver.source.bb.arbitrary.wsegment.configure.get_comment() \n
		This command enters a comment for the configuration file. The configuration file must be specified with command
		BB:ARB:WSEG:CONF:SEL. \n
			:return: comment: string
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:WSEGment:CONFigure:COMMent?')
		return trim_str_response(response)

	def set_comment(self, comment: str) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WSEGment:CONFigure:COMMent \n
		Snippet: driver.source.bb.arbitrary.wsegment.configure.set_comment(comment = '1') \n
		This command enters a comment for the configuration file. The configuration file must be specified with command
		BB:ARB:WSEG:CONF:SEL. \n
			:param comment: string
		"""
		param = Conversions.value_to_quoted_str(comment)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:WSEGment:CONFigure:COMMent {param}')

	def delete(self, filename: str) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WSEGment:CONFigure:DELete \n
		Snippet: driver.source.bb.arbitrary.wsegment.configure.delete(filename = '1') \n
		Deletes the configuration file. The configuration files are used to create multi segment waveform files. \n
			:param filename: string
		"""
		param = Conversions.value_to_quoted_str(filename)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:WSEGment:CONFigure:DELete {param}')

	def get_ofile(self) -> str:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WSEGment:CONFigure:OFILe \n
		Snippet: value: str = driver.source.bb.arbitrary.wsegment.configure.get_ofile() \n
		Defines the file name for the multi segment waveform. The file extension is *.wv. \n
			:return: ofile: string
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:WSEGment:CONFigure:OFILe?')
		return trim_str_response(response)

	def set_ofile(self, ofile: str) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WSEGment:CONFigure:OFILe \n
		Snippet: driver.source.bb.arbitrary.wsegment.configure.set_ofile(ofile = '1') \n
		Defines the file name for the multi segment waveform. The file extension is *.wv. \n
			:param ofile: string
		"""
		param = Conversions.value_to_quoted_str(ofile)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:WSEGment:CONFigure:OFILe {param}')

	def get_select(self) -> str:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WSEGment:CONFigure:SELect \n
		Snippet: value: str = driver.source.bb.arbitrary.wsegment.configure.get_select() \n
		Selects the configuration file in the default directory. A path can also be specified, in which case the files in the
		specified directory are selected. If a configuration file with the specified name does not yet exist, it is created. The
		file extension may be omitted. Only files with the file extension *.inf_mswv will be selected or created. \n
			:return: filename: string
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:WSEGment:CONFigure:SELect?')
		return trim_str_response(response)

	def set_select(self, filename: str) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WSEGment:CONFigure:SELect \n
		Snippet: driver.source.bb.arbitrary.wsegment.configure.set_select(filename = '1') \n
		Selects the configuration file in the default directory. A path can also be specified, in which case the files in the
		specified directory are selected. If a configuration file with the specified name does not yet exist, it is created. The
		file extension may be omitted. Only files with the file extension *.inf_mswv will be selected or created. \n
			:param filename: string
		"""
		param = Conversions.value_to_quoted_str(filename)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:WSEGment:CONFigure:SELect {param}')

	def clone(self) -> 'ConfigureCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = ConfigureCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class RequestCls:
	"""Request commands group definition. 2 total commands, 1 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("request", core, parent)

	@property
	def shared(self):
		"""shared commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_shared'):
			from .Shared import SharedCls
			self._shared = SharedCls(self._core, self._cmd_group)
		return self._shared

	def get_exclusive(self) -> int:
		"""SCPI: SYSTem:LOCK:REQuest:[EXCLusive] \n
		Snippet: value: int = driver.system.lock.request.get_exclusive() \n
		No command help available \n
			:return: success: No help available
		"""
		response = self._core.io.query_str('SYSTem:LOCK:REQuest:EXCLusive?')
		return Conversions.str_to_int(response)

	def clone(self) -> 'RequestCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = RequestCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group
